import opcodetools.cpu.base_cpu


OPCODES = [
    {"mnemonic": "BRK",              "code": "00",               "use": ""             },
    {"mnemonic": "ORA (p,X)",        "code": "01pp",             "use": "p=data_r"     },
    {"mnemonic": "ORA p",            "code": "05pp",             "use": "p=data_r"     },
    {"mnemonic": "ASL p",            "code": "06pp",             "use": "p=data_rw"    },
    {"mnemonic": "PHP",              "code": "08",               "use": ""             },
    {"mnemonic": "ORA #b",           "code": "09bb",             "use": "b=const"      },
    {"mnemonic": "ASL A",            "code": "0A",               "use": ""             },
    {"mnemonic": "ORA t",            "code": "0Dt0t1",           "use": "t=data_r"     },
    {"mnemonic": "ASL t",            "code": "0Et0t1",           "use": "t=data_rw"    },
    {"mnemonic": "BPL r",            "code": "10rr",             "use": "r=code_pcr"   },
    {"mnemonic": "ORA (p),Y",        "code": "11pp",             "use": "p=data_r"     },
    {"mnemonic": "ORA p,X",          "code": "15pp",             "use": "p=data_r"     },
    {"mnemonic": "ASL p,X",          "code": "16pp",             "use": "p=data_rw"    },
    {"mnemonic": "CLC",              "code": "18",               "use": ""             },
    {"mnemonic": "ORA t,Y",          "code": "19t0t1",           "use": "t=data_r"     },
    {"mnemonic": "ORA t,X",          "code": "1Dt0t1",           "use": "t=data_r"     },
    {"mnemonic": "ASL t,X",          "code": "1Et0t1",           "use": "t=data_rw"    },
    {"mnemonic": "JSR t",            "code": "20t0t1",           "use": "t=code"       },
    {"mnemonic": "AND (p,X)",        "code": "21pp",             "use": "p=data_r"     },
    {"mnemonic": "BIT p",            "code": "24pp",             "use": "p=data_r"     },
    {"mnemonic": "AND p",            "code": "25pp",             "use": "p=data_r"     },
    {"mnemonic": "ROL p",            "code": "26pp",             "use": "p=data_rw"    },
    {"mnemonic": "PLP",              "code": "28",               "use": ""             },
    {"mnemonic": "AND #b",           "code": "29bb",             "use": "b=const"      },
    {"mnemonic": "ROL A",            "code": "2A",               "use": "",             },
    {"mnemonic": "BIT t",            "code": "2Ct0t1",           "use": "t=data_r"     },
    {"mnemonic": "AND t",            "code": "2Dt0t1",           "use": "t=data_r"     },
    {"mnemonic": "ROL t",            "code": "2Et0t1",           "use": "t=data_rw"    },
    {"mnemonic": "BMI r",            "code": "30rr",             "use": "r=code_pcr"   },
    {"mnemonic": "AND (p),Y",        "code": "31pp",             "use": "p=data_r"     },
    {"mnemonic": "AND p,X",          "code": "35pp",             "use": "p=data_r"     },
    {"mnemonic": "ROL p,X",          "code": "36pp",             "use": "p=data_rw"    },
    {"mnemonic": "SEC",              "code": "38",               "use": ""             },
    {"mnemonic": "AND t,Y",          "code": "39t0t1",           "use": "t=data_r"     },
    {"mnemonic": "AND t,X",          "code": "3Dt0t1",           "use": "t=data_r"     },
    {"mnemonic": "ROL t,X",          "code": "3Et0t1",           "use": "t=data_rw"    },
    {"mnemonic": "RTI",              "code": "40",               "use": ""             },
    {"mnemonic": "EOR (p,X)",        "code": "41pp",             "use": "p=data_r"     },
    {"mnemonic": "EOR p",            "code": "45pp",             "use": "p=data_r"     },
    {"mnemonic": "LSR p",            "code": "46pp",             "use": "p=data_rw"    },
    {"mnemonic": "PHA",              "code": "48",               "use": ""             },
    {"mnemonic": "EOR #b",           "code": "49bb",             "use": "b=const"      },
    {"mnemonic": "LSR A",            "code": "4A",               "use": ""             },
    {"mnemonic": "JMP t",            "code": "4Ct0t1",           "use": "t=code"       },
    {"mnemonic": "EOR t",            "code": "4Dt0t1",           "use": "t=data_r"     },
    {"mnemonic": "LSR t",            "code": "4Et0t1",           "use": "t=data_rw"    },
    {"mnemonic": "BVC r",            "code": "50rr",             "use": "r=code_pcr"   },
    {"mnemonic": "EOR (p),Y",        "code": "51pp",             "use": "p=data_r"     },
    {"mnemonic": "EOR p,X",          "code": "55pp",             "use": "p=data_r"     },
    {"mnemonic": "LSR p,X",          "code": "56pp",             "use": "p=data_r"     },
    {"mnemonic": "CLI",              "code": "58",               "use": ""             },
    {"mnemonic": "EOR t,Y",          "code": "59t0t1",           "use": "t=data_r"     },
    {"mnemonic": "EOR t,X",          "code": "5Dt0t1",           "use": "t=data_r"     },
    {"mnemonic": "LSR t,X",          "code": "5Et0t1",           "use": "t=data_r"     },
    {"mnemonic": "RTS",              "code": "60",               "use": ""             },
    {"mnemonic": "ADC (p,X)",        "code": "61pp",             "use": "p=data_r"     },
    {"mnemonic": "ADC p",            "code": "65pp",             "use": "p=data_r"     },
    {"mnemonic": "ROR p",            "code": "66pp",             "use": "p=data_rw"    },
    {"mnemonic": "PLA",              "code": "68",               "use": ""             },
    {"mnemonic": "ADC #b",           "code": "69bb",             "use": "b=const"      },
    {"mnemonic": "ROR A",            "code": "6A",               "use": ""             },
    {"mnemonic": "JMP (t)",          "code": "6Ct0t1",           "use": "t=code"       },
    {"mnemonic": "ADC t",            "code": "6Dt0t1",           "use": "t=data_r"     },
    {"mnemonic": "ROR t",            "code": "6Et0t1",           "use": "t=data_rw"    },
    {"mnemonic": "BVS r",            "code": "70rr",             "use": "r=code_pcr"   },
    {"mnemonic": "ADC (p),Y",        "code": "71pp",             "use": "p=data_r"     },
    {"mnemonic": "ADC p,X",          "code": "75pp",             "use": "p=data_r"     },
    {"mnemonic": "ROR p,X",          "code": "76pp",             "use": "p=data_r"     },
    {"mnemonic": "SEI",              "code": "78",               "use": ""             },
    {"mnemonic": "ADC t,Y",          "code": "79t0t1",           "use": "t=data_r"     },
    {"mnemonic": "ADC t,X",          "code": "7Dt0t1",           "use": "t=data_r"     },
    {"mnemonic": "ROR t,X",          "code": "7Et0t1",           "use": "t=data_rw"    },
    {"mnemonic": "STA (p,X)",        "code": "81pp",             "use": "p=data_w"     },
    {"mnemonic": "STY p",            "code": "84pp",             "use": "p=data_w"     },
    {"mnemonic": "STA p",            "code": "85pp",             "use": "p=data_w"     },
    {"mnemonic": "STX p",            "code": "86pp",             "use": "p=data_w"     },
    {"mnemonic": "DEY",              "code": "88",               "use": ""             },
    {"mnemonic": "TXA",              "code": "8A",               "use": ""             },
    {"mnemonic": "STY t",            "code": "8Ct0t1",           "use": "t=data_w"     },
    {"mnemonic": "STA t",            "code": "8Dt0t1",           "use": "t=data_w"     },
    {"mnemonic": "STX t",            "code": "8Et0t1",           "use": "t=data_w"     },
    {"mnemonic": "BCC r",            "code": "90rr",             "use": "r=code_pcr"   },
    {"mnemonic": "STA (p),Y",        "code": "91pp",             "use": "p=data_w"     },
    {"mnemonic": "STY p,X",          "code": "94pp",             "use": "p=data_w"     },
    {"mnemonic": "STA p,X",          "code": "95pp",             "use": "p=data_w"     },
    {"mnemonic": "STX p,Y",          "code": "96pp",             "use": "p=data_w"     },
    {"mnemonic": "TYA",              "code": "98",               "use": ""             },
    {"mnemonic": "STA t,Y",          "code": "99t0t1",           "use": "t=data_w"     },
    {"mnemonic": "TXS",              "code": "9A",               "use": ""             },
    {"mnemonic": "STA t,X",          "code": "9Dt0t1",           "use": "t=data_w"     },
    {"mnemonic": "LDY #b",           "code": "A0bb",             "use": "b=const"      },
    {"mnemonic": "LDA (p,X)",        "code": "A1pp",             "use": "p=data_r"     },
    {"mnemonic": "LDX #b",           "code": "A2bb",             "use": "b=const"      },
    {"mnemonic": "LDY p",            "code": "A4pp",             "use": "p=data_r"     },
    {"mnemonic": "LDA p",            "code": "A5pp",             "use": "p=data_r"     },
    {"mnemonic": "LDX p",            "code": "A6pp",             "use": "p=data_r"     },
    {"mnemonic": "TAY",              "code": "A8",               "use": ""             },
    {"mnemonic": "LDA #b",           "code": "A9bb",             "use": "b=const"      },
    {"mnemonic": "TAX",              "code": "AA",               "use": ""             },
    {"mnemonic": "LDY t",            "code": "ACt0t1",           "use": "t=data_r"     },
    {"mnemonic": "LDA t",            "code": "ADt0t1",           "use": "t=data_r"     },
    {"mnemonic": "LDX t",            "code": "AEt0t1",           "use": "t=data_r"     },
    {"mnemonic": "BCS r",            "code": "B0rr",             "use": "r=code_pcr"   },
    {"mnemonic": "LDA (p),Y",        "code": "B1pp",             "use": "p=data_r"     },
    {"mnemonic": "LDY p,X",          "code": "B4pp",             "use": "p=data_r"     },
    {"mnemonic": "LDA p,X",          "code": "B5pp",             "use": "p=data_r"     },
    {"mnemonic": "LDX p,Y",          "code": "B6pp",             "use": "p=data_r"     },
    {"mnemonic": "CLV",              "code": "B8",               "use": ""             },
    {"mnemonic": "LDA t,Y",          "code": "B9t0t1",           "use": "t=data_r"     },
    {"mnemonic": "TSX",              "code": "BA",               "use": ""             },
    {"mnemonic": "LDY t,X",          "code": "BCt0t1",           "use": "t=data_r"     },
    {"mnemonic": "LDA t,X",          "code": "BDt0t1",           "use": "t=data_r"     },
    {"mnemonic": "LDX t,Y",          "code": "BEt0t1",           "use": "t=data_r"     },
    {"mnemonic": "CPY #b",           "code": "C0bb",             "use": "b=const"      },
    {"mnemonic": "CMP (p,X)",        "code": "C1pp",             "use": "p=data_r"     },
    {"mnemonic": "CPY p",            "code": "C4pp",             "use": "p=data_r"     },
    {"mnemonic": "CMP p",            "code": "C5pp",             "use": "p=data_r"     },
    {"mnemonic": "DEC p",            "code": "C6pp",             "use": "p=data_rw"    },
    {"mnemonic": "INY",              "code": "C8",               "use": ""             },
    {"mnemonic": "CMP #b",           "code": "C9bb",             "use": "b=const"      },
    {"mnemonic": "DEX",              "code": "CA",               "use": ""             },
    {"mnemonic": "CPY t",            "code": "CCt0t1",           "use": "t=data_r"     },
    {"mnemonic": "CMP t",            "code": "CDt0t1",           "use": "t=data_r"     },
    {"mnemonic": "DEC t",            "code": "CEt0t1",           "use": "t=data_rw"    },
    {"mnemonic": "BNE r",            "code": "D0rr",             "use": "r=code_pcr"   },
    {"mnemonic": "CMP (p),Y",        "code": "D1pp",             "use": "p=data_r"     },
    {"mnemonic": "CMP p,X",          "code": "D5pp",             "use": "p=data_r"     },
    {"mnemonic": "DEC p,X",          "code": "D6pp",             "use": "p=data_rw"    },
    {"mnemonic": "CLD",              "code": "D8",               "use": ""             },
    {"mnemonic": "CMP t,Y",          "code": "D9t0t1",           "use": "t=data_r"     },
    {"mnemonic": "CMP t,X",          "code": "DDt0t1",           "use": "t=data_r"     },
    {"mnemonic": "DEC t,X",          "code": "DEt0t1",           "use": "t=data_rw"    },
    {"mnemonic": "CPX #b",           "code": "E0bb",             "use": "b=const"      },
    {"mnemonic": "SBC (p,X)",        "code": "E1pp",             "use": "p=data_r"     },
    {"mnemonic": "CPX p",            "code": "E4pp",             "use": "p=data_r"     },
    {"mnemonic": "SBC p",            "code": "E5pp",             "use": "p=data_r"     },
    {"mnemonic": "INC p",            "code": "E6pp",             "use": "p=data_rw"    },
    {"mnemonic": "INX",              "code": "E8",               "use": ""             },
    {"mnemonic": "SBC #b",           "code": "E9bb",             "use": "b=const"      },
    {"mnemonic": "NOP",              "code": "EA",               "use": ""             },
    {"mnemonic": "CPX t",            "code": "ECt0t1",           "use": "t=data_r"     },
    {"mnemonic": "SBC t",            "code": "EDt0t1",           "use": "t=data_r"     },
    {"mnemonic": "INC t",            "code": "EEt0t1",           "use": "t=data_rw"    },
    {"mnemonic": "BEQ r",            "code": "F0rr",             "use": "r=code_pcr"   },
    {"mnemonic": "SBC (p),Y",        "code": "F1pp",             "use": "p=data_r"     },
    {"mnemonic": "SBC p,X",          "code": "F5pp",             "use": "p=data_r"     },
    {"mnemonic": "INC p,X",          "code": "F6pp",             "use": "p=data_rw"    },
    {"mnemonic": "SED",              "code": "F8",               "use": ""             },
    {"mnemonic": "SBC t,Y",          "code": "F9t0t1",           "use": "t=data_r"     },
    {"mnemonic": "SBC t,X",          "code": "FDt0t1",           "use": "t=data_r"     },
    {"mnemonic": "INC t,X",          "code": "FEt0t1",           "use": "t=data_rw"    },
]


class CPU_6502(opcodetools.cpu.base_cpu.CPU):

    def __init__(self):
        super().__init__(OPCODES)




fname = 'cpu_Z80GB.py'

with open(fname) as f:
    lines = f.readlines()

for o in lines:
    o = o[0:-1]
    if o.endswith('},'):
        x = len(o) - 3
        while o[x] == ' ':
            x -= 1
        if o[x] == ',':
            o = o[:x] + o[x + 1:]
    print(o)



import opcodetools.cpu.base_cpu


OPCODES = [
    {"mnemonic": "ADC A,A",          "code": "8F",               "use": ""             },
    {"mnemonic": "ADC A,B",          "code": "88",               "use": ""             },
    {"mnemonic": "ADC A,C",          "code": "89",               "use": ""             },
    {"mnemonic": "ADC A,D",          "code": "8A",               "use": ""             },
    {"mnemonic": "ADC A,E",          "code": "8B",               "use": ""             },
    {"mnemonic": "ADC A,H",          "code": "8C",               "use": ""             },
    {"mnemonic": "ADC A,L",          "code": "8D",               "use": ""             },
    {"mnemonic": "ADC A,(HL)",       "code": "8E",               "use": ""             },
    {"mnemonic": "ADC A,(IX+i)",     "code": "DD8Eii",           "use": "i=const"      },
    {"mnemonic": "ADC A,(IY+i)",     "code": "FD8Eii",           "use": "i=const"      },
    {"mnemonic": "ADC HL,BC",        "code": "ED4A",             "use": ""             },
    {"mnemonic": "ADC HL,DE",        "code": "ED5A",             "use": ""             },
    {"mnemonic": "ADC HL,HL",        "code": "ED6A",             "use": ""             },
    {"mnemonic": "ADC HL,SP",        "code": "ED7A",             "use": ""             },
    {"mnemonic": "ADC b",            "code": "CEbb",             "use": "b=const"      },
    {"mnemonic": "ADD IX,BC",        "code": "DD09",             "use": ""             },
    {"mnemonic": "ADD IX,DE",        "code": "DD19",             "use": ""             },
    {"mnemonic": "ADD IX,IX",        "code": "DD29",             "use": ""             },
    {"mnemonic": "ADD IX,SP",        "code": "DD39",             "use": ""             },
    {"mnemonic": "ADD IY,BC",        "code": "FD09",             "use": ""             },
    {"mnemonic": "ADD IY,DE",        "code": "FD19",             "use": ""             },
    {"mnemonic": "ADD IY,IY",        "code": "FD29",             "use": ""             },
    {"mnemonic": "ADD IY,SP",        "code": "FD39",             "use": ""             },
    {"mnemonic": "ADD A,A",          "code": "87",               "use": ""             },
    {"mnemonic": "ADD A,B",          "code": "80",               "use": ""             },
    {"mnemonic": "ADD A,C",          "code": "81",               "use": ""             },
    {"mnemonic": "ADD A,D",          "code": "82",               "use": ""             },
    {"mnemonic": "ADD A,E",          "code": "83",               "use": ""             },
    {"mnemonic": "ADD A,H",          "code": "84",               "use": ""             },
    {"mnemonic": "ADD A,L",          "code": "85",               "use": ""             },
    {"mnemonic": "ADD A,IXL",        "code": "DD85",             "use": ""             },
    {"mnemonic": "ADD A,(HL)",       "code": "86",               "use": ""             },
    {"mnemonic": "ADD A,(IX+i)",     "code": "DD86ii",           "use": "i=const"      },
    {"mnemonic": "ADD A,(IY+i)",     "code": "FD86ii",           "use": "i=const"      },
    {"mnemonic": "ADD HL,BC",        "code": "09",               "use": ""             },
    {"mnemonic": "ADD HL,DE",        "code": "19",               "use": ""             },
    {"mnemonic": "ADD HL,HL",        "code": "29",               "use": ""             },
    {"mnemonic": "ADD HL,SP",        "code": "39",               "use": ""             },
    {"mnemonic": "ADD b",            "code": "C6bb",             "use": "b=const"      },
    {"mnemonic": "AND A",            "code": "A7",               "use": ""             },
    {"mnemonic": "AND B",            "code": "A0",               "use": ""             },
    {"mnemonic": "AND C",            "code": "A1",               "use": ""             },
    {"mnemonic": "AND D",            "code": "A2",               "use": ""             },
    {"mnemonic": "AND E",            "code": "A3",               "use": ""             },
    {"mnemonic": "AND H",            "code": "A4",               "use": ""             },
    {"mnemonic": "AND L",            "code": "A5",               "use": ""             },
    {"mnemonic": "AND (HL)",         "code": "A6",               "use": ""             },
    {"mnemonic": "AND (IX+i)",       "code": "DDA6ii",           "use": "i=const"      },
    {"mnemonic": "AND (IY+i)",       "code": "FDA6ii",           "use": "i=const"      },
    {"mnemonic": "AND b",            "code": "E6bb",             "use": "b=const"      },
    {"mnemonic": "OR A",             "code": "B7",               "use": ""             },
    {"mnemonic": "OR B",             "code": "B0",               "use": ""             },
    {"mnemonic": "OR C",             "code": "B1",               "use": ""             },
    {"mnemonic": "OR D",             "code": "B2",               "use": ""             },
    {"mnemonic": "OR E",             "code": "B3",               "use": ""             },
    {"mnemonic": "OR H",             "code": "B4",               "use": ""             },
    {"mnemonic": "OR L",             "code": "B5",               "use": ""             },
    {"mnemonic": "OR (HL)",          "code": "B6",               "use": ""             },
    {"mnemonic": "OR (IX+i)",        "code": "DDB6ii",           "use": "i=const"      },
    {"mnemonic": "OR (IY+i)",        "code": "FDB6ii",           "use": "i=const"      },
    {"mnemonic": "OR b",             "code": "F6bb",             "use": "b=const"      },
    {"mnemonic": "XOR A",            "code": "AF",               "use": ""             },
    {"mnemonic": "XOR B",            "code": "A8",               "use": ""             },
    {"mnemonic": "XOR C",            "code": "A9",               "use": ""             },
    {"mnemonic": "XOR D",            "code": "AA",               "use": ""             },
    {"mnemonic": "XOR E",            "code": "AB",               "use": ""             },
    {"mnemonic": "XOR H",            "code": "AC",               "use": ""             },
    {"mnemonic": "XOR L",            "code": "AD",               "use": ""             },
    {"mnemonic": "XOR (HL)",         "code": "AE",               "use": ""             },
    {"mnemonic": "XOR (IX+i)",       "code": "DDAEii",           "use": "i=const"      },
    {"mnemonic": "XOR (IY+i)",       "code": "FDAEii",           "use": "i=const"      },
    {"mnemonic": "XOR b",            "code": "EEbb",             "use": "b=const"      },
    {"mnemonic": "BIT 0,A",          "code": "CB47",             "use": ""             },
    {"mnemonic": "BIT 0,B",          "code": "CB40",             "use": ""             },
    {"mnemonic": "BIT 0,C",          "code": "CB41",             "use": ""             },
    {"mnemonic": "BIT 0,D",          "code": "CB42",             "use": ""             },
    {"mnemonic": "BIT 0,E",          "code": "CB43",             "use": ""             },
    {"mnemonic": "BIT 0,H",          "code": "CB44",             "use": ""             },
    {"mnemonic": "BIT 0,L",          "code": "CB45",             "use": ""             },
    {"mnemonic": "BIT 1,A",          "code": "CB4F",             "use": ""             },
    {"mnemonic": "BIT 1,B",          "code": "CB48",             "use": ""             },
    {"mnemonic": "BIT 1,C",          "code": "CB49",             "use": ""             },
    {"mnemonic": "BIT 1,D",          "code": "CB4A",             "use": ""             },
    {"mnemonic": "BIT 1,E",          "code": "CB4B",             "use": ""             },
    {"mnemonic": "BIT 1,H",          "code": "CB4C",             "use": ""             },
    {"mnemonic": "BIT 1,L",          "code": "CB4D",             "use": ""             },
    {"mnemonic": "BIT 2,A",          "code": "CB57",             "use": ""             },
    {"mnemonic": "BIT 2,B",          "code": "CB50",             "use": ""             },
    {"mnemonic": "BIT 2,C",          "code": "CB51",             "use": ""             },
    {"mnemonic": "BIT 2,D",          "code": "CB52",             "use": ""             },
    {"mnemonic": "BIT 2,E",          "code": "CB53",             "use": ""             },
    {"mnemonic": "BIT 2,H",          "code": "CB54",             "use": ""             },
    {"mnemonic": "BIT 2,L",          "code": "CB55",             "use": ""             },
    {"mnemonic": "BIT 3,A",          "code": "CB5F",             "use": ""             },
    {"mnemonic": "BIT 3,B",          "code": "CB58",             "use": ""             },
    {"mnemonic": "BIT 3,C",          "code": "CB59",             "use": ""             },
    {"mnemonic": "BIT 3,D",          "code": "CB5A",             "use": ""             },
    {"mnemonic": "BIT 3,E",          "code": "CB5B",             "use": ""             },
    {"mnemonic": "BIT 3,H",          "code": "CB5C",             "use": ""             },
    {"mnemonic": "BIT 3,L",          "code": "CB5D",             "use": ""             },
    {"mnemonic": "BIT 4,A",          "code": "CB67",             "use": ""             },
    {"mnemonic": "BIT 4,B",          "code": "CB60",             "use": ""             },
    {"mnemonic": "BIT 4,C",          "code": "CB61",             "use": ""             },
    {"mnemonic": "BIT 4,D",          "code": "CB62",             "use": ""             },
    {"mnemonic": "BIT 4,E",          "code": "CB63",             "use": ""             },
    {"mnemonic": "BIT 4,H",          "code": "CB64",             "use": ""             },
    {"mnemonic": "BIT 4,L",          "code": "CB65",             "use": ""             },
    {"mnemonic": "BIT 5,A",          "code": "CB6F",             "use": ""             },
    {"mnemonic": "BIT 5,B",          "code": "CB68",             "use": ""             },
    {"mnemonic": "BIT 5,C",          "code": "CB69",             "use": ""             },
    {"mnemonic": "BIT 5,D",          "code": "CB6A",             "use": ""             },
    {"mnemonic": "BIT 5,E",          "code": "CB6B",             "use": ""             },
    {"mnemonic": "BIT 5,H",          "code": "CB6C",             "use": ""             },
    {"mnemonic": "BIT 5,L",          "code": "CB6D",             "use": ""             },
    {"mnemonic": "BIT 6,A",          "code": "CB77",             "use": ""             },
    {"mnemonic": "BIT 6,B",          "code": "CB70",             "use": ""             },
    {"mnemonic": "BIT 6,C",          "code": "CB71",             "use": ""             },
    {"mnemonic": "BIT 6,D",          "code": "CB72",             "use": ""             },
    {"mnemonic": "BIT 6,E",          "code": "CB73",             "use": ""             },
    {"mnemonic": "BIT 6,H",          "code": "CB74",             "use": ""             },
    {"mnemonic": "BIT 6,L",          "code": "CB75",             "use": ""             },
    {"mnemonic": "BIT 7,A",          "code": "CB7F",             "use": ""             },
    {"mnemonic": "BIT 7,B",          "code": "CB78",             "use": ""             },
    {"mnemonic": "BIT 7,C",          "code": "CB79",             "use": ""             },
    {"mnemonic": "BIT 7,D",          "code": "CB7A",             "use": ""             },
    {"mnemonic": "BIT 7,E",          "code": "CB7B",             "use": ""             },
    {"mnemonic": "BIT 7,H",          "code": "CB7C",             "use": ""             },
    {"mnemonic": "BIT 7,L",          "code": "CB7D",             "use": ""             },
    {"mnemonic": "BIT 0,(HL)",       "code": "CB46",             "use": ""             },
    {"mnemonic": "BIT 1,(HL)",       "code": "CB4E",             "use": ""             },
    {"mnemonic": "BIT 2,(HL)",       "code": "CB56",             "use": ""             },
    {"mnemonic": "BIT 3,(HL)",       "code": "CB5E",             "use": ""             },
    {"mnemonic": "BIT 4,(HL)",       "code": "CB66",             "use": ""             },
    {"mnemonic": "BIT 5,(HL)",       "code": "CB6E",             "use": ""             },
    {"mnemonic": "BIT 6,(HL)",       "code": "CB76",             "use": ""             },
    {"mnemonic": "BIT 7,(HL)",       "code": "CB7E",             "use": ""             },
    {"mnemonic": "BIT 0,(IX+i)",     "code": "DDCBii46",         "use": "i=const"      },
    {"mnemonic": "BIT 0,(IY+i)",     "code": "FDCBii46",         "use": "i=const"      },
    {"mnemonic": "BIT 1,(IX+i)",     "code": "DDCBii4E",         "use": "i=const"      },
    {"mnemonic": "BIT 1,(IY+i)",     "code": "FDCBii4E",         "use": "i=const"      },
    {"mnemonic": "BIT 2,(IX+i)",     "code": "DDCBii56",         "use": "i=const"      },
    {"mnemonic": "BIT 2,(IY+i)",     "code": "FDCBii56",         "use": "i=const"      },
    {"mnemonic": "BIT 3,(IX+i)",     "code": "DDCBii5E",         "use": "i=const"      },
    {"mnemonic": "BIT 3,(IY+i)",     "code": "FDCBii5E",         "use": "i=const"      },
    {"mnemonic": "BIT 4,(IX+i)",     "code": "DDCBii66",         "use": "i=const"      },
    {"mnemonic": "BIT 4,(IY+i)",     "code": "FDCBii66",         "use": "i=const"      },
    {"mnemonic": "BIT 5,(IX+i)",     "code": "DDCBii6E",         "use": "i=const"      },
    {"mnemonic": "BIT 5,(IY+i)",     "code": "FDCBii6E",         "use": "i=const"      },
    {"mnemonic": "BIT 6,(IX+i)",     "code": "DDCBii76",         "use": "i=const"      },
    {"mnemonic": "BIT 6,(IY+i)",     "code": "FDCBii76",         "use": "i=const"      },
    {"mnemonic": "BIT 7,(IX+i)",     "code": "DDCBii7E",         "use": "i=const"      },
    {"mnemonic": "BIT 7,(IY+i)",     "code": "FDCBii7E",         "use": "i=const"      },
    {"mnemonic": "CALL t",           "code": "CDt0t1",           "use": "t=code"       },
    {"mnemonic": "CALL NZ,t",        "code": "C4t0t1",           "use": "t=code"       },
    {"mnemonic": "CALL Z,t",         "code": "CCt0t1",           "use": "t=code"       },
    {"mnemonic": "CALL NC,t",        "code": "D4t0t1",           "use": "t=code"       },
    {"mnemonic": "CALL C,t",         "code": "DCt0t1",           "use": "t=code"       },
    {"mnemonic": "CALL C,t",         "code": "FDDCt0t1",         "use": "t=code"       },
    {"mnemonic": "CALL PO,t",        "code": "E4t0t1",           "use": "t=code"       },
    {"mnemonic": "CALL PE,t",        "code": "ECt0t1",           "use": "t=code"       },
    {"mnemonic": "CALL P,t",         "code": "F4t0t1",           "use": "t=code"       },
    {"mnemonic": "CALL M,t",         "code": "FCt0t1",           "use": "t=code"       },
    {"mnemonic": "CCF",              "code": "3F",               "use": ""             },
    {"mnemonic": "CP A",             "code": "BF",               "use": ""             },
    {"mnemonic": "CP B",             "code": "B8",               "use": ""             },
    {"mnemonic": "CP C",             "code": "B9",               "use": ""             },
    {"mnemonic": "CP D",             "code": "BA",               "use": ""             },
    {"mnemonic": "CP E",             "code": "BB",               "use": ""             },
    {"mnemonic": "CP H",             "code": "BC",               "use": ""             },
    {"mnemonic": "CP L",             "code": "BD",               "use": ""             },
    {"mnemonic": "CP (HL)",          "code": "BE",               "use": ""             },
    {"mnemonic": "CP (IX+i)",        "code": "DDBEii",           "use": "i=const"      },
    {"mnemonic": "CP (IY+i)",        "code": "FDBEii",           "use": "i=const"      },
    {"mnemonic": "CP b",             "code": "FEbb",             "use": "b=const"      },
    {"mnemonic": "CPD",              "code": "EDA9",             "use": ""             },
    {"mnemonic": "CPI",              "code": "EDA1",             "use": ""             },
    {"mnemonic": "CPDR",             "code": "EDB9",             "use": ""             },
    {"mnemonic": "CPIR",             "code": "EDB1",             "use": ""             },
    {"mnemonic": "DAA",              "code": "27",               "use": ""             },
    {"mnemonic": "DI",               "code": "F3",               "use": ""             },
    {"mnemonic": "EI",               "code": "FB",               "use": ""             },
    {"mnemonic": "DJNZ r",           "code": "10rr",             "use": "r=code_pcr"   },
    {"mnemonic": "EX AF,AF'",        "code": "08",               "use": ""             },
    {"mnemonic": "EXX",              "code": "D9",               "use": ""             },
    {"mnemonic": "HALT",             "code": "76",               "use": ""             },
    {"mnemonic": "IM 0",             "code": "ED46",             "use": ""             },
    {"mnemonic": "IM 1",             "code": "ED56",             "use": ""             },
    {"mnemonic": "IM 2",             "code": "ED5E",             "use": ""             },
    {"mnemonic": "IND",              "code": "EDAA",             "use": ""             },
    {"mnemonic": "INDR",             "code": "EDBA",             "use": ""             },
    {"mnemonic": "INI",              "code": "EDA2",             "use": ""             },
    {"mnemonic": "INIR",             "code": "EDB2",             "use": ""             },
    {"mnemonic": "LDDR",             "code": "EDB8",             "use": ""             },
    {"mnemonic": "LDIR",             "code": "EDB0",             "use": ""             },
    {"mnemonic": "OTDR",             "code": "EDBB",             "use": ""             },
    {"mnemonic": "OTIR",             "code": "EDB3",             "use": ""             },
    {"mnemonic": "OUTD",             "code": "EDAB",             "use": ""             },
    {"mnemonic": "OUTI",             "code": "EDA3",             "use": ""             },
    {"mnemonic": "RETI",             "code": "ED4D",             "use": ""             },
    {"mnemonic": "RETN",             "code": "ED45",             "use": ""             },
    {"mnemonic": "RLD",              "code": "ED6F",             "use": ""             },
    {"mnemonic": "RRD",              "code": "ED67",             "use": ""             },
    {"mnemonic": "CPL",              "code": "2F",               "use": ""             },
    {"mnemonic": "DEC A",            "code": "3D",               "use": ""             },
    {"mnemonic": "DEC B",            "code": "05",               "use": ""             },
    {"mnemonic": "DEC C",            "code": "0D",               "use": ""             },
    {"mnemonic": "DEC D",            "code": "15",               "use": ""             },
    {"mnemonic": "DEC E",            "code": "1D",               "use": ""             },
    {"mnemonic": "DEC H",            "code": "25",               "use": ""             },
    {"mnemonic": "DEC L",            "code": "2D",               "use": ""             },
    {"mnemonic": "DEC (HL)",         "code": "35",               "use": ""             },
    {"mnemonic": "DEC (IX+i)",       "code": "DD35ii",           "use": "i=constw"     },
    {"mnemonic": "DEC (IY+i)",       "code": "FD35ii",           "use": "i=constw"     },
    {"mnemonic": "DEC BC",           "code": "0B",               "use": ""             },
    {"mnemonic": "DEC DE",           "code": "1B",               "use": ""             },
    {"mnemonic": "DEC HL",           "code": "2B",               "use": ""             },
    {"mnemonic": "DEC SP",           "code": "3B",               "use": ""             },
    {"mnemonic": "DEC IX",           "code": "DD2B",             "use": ""             },
    {"mnemonic": "DEC IY",           "code": "FD2B",             "use": ""             },
    {"mnemonic": "EX DE,HL",         "code": "EB",               "use": ""             },
    {"mnemonic": "EX (SP),HL",       "code": "E3",               "use": ""             },
    {"mnemonic": "EX (SP),IX",       "code": "DDE3",             "use": ""             },
    {"mnemonic": "EX (SP),IY",       "code": "FDE3",             "use": ""             },
    {"mnemonic": "IN A,(C)",         "code": "ED78",             "use": ""             },
    {"mnemonic": "IN B,(C)",         "code": "ED40",             "use": ""             },
    {"mnemonic": "IN C,(C)",         "code": "ED48",             "use": ""             },
    {"mnemonic": "IN D,(C)",         "code": "ED50",             "use": ""             },
    {"mnemonic": "IN E,(C)",         "code": "ED58",             "use": ""             },
    {"mnemonic": "IN H,(C)",         "code": "ED60",             "use": ""             },
    {"mnemonic": "IN L,(C)",         "code": "ED68",             "use": ""             },
    {"mnemonic": "IN A,(o)",         "code": "DBoo",             "use": "o=port_r"     },
    {"mnemonic": "INC A",            "code": "3C",               "use": ""             },
    {"mnemonic": "INC B",            "code": "04",               "use": ""             },
    {"mnemonic": "INC C",            "code": "0C",               "use": ""             },
    {"mnemonic": "INC D",            "code": "14",               "use": ""             },
    {"mnemonic": "INC E",            "code": "1C",               "use": ""             },
    {"mnemonic": "INC H",            "code": "24",               "use": ""             },
    {"mnemonic": "INC L",            "code": "2C",               "use": ""             },
    {"mnemonic": "INC (HL)",         "code": "34",               "use": ""             },
    {"mnemonic": "INC (IX+i)",       "code": "DD34ii",           "use": "i=constw"     },
    {"mnemonic": "INC (IY+i)",       "code": "FD34ii",           "use": "i=constw"     },
    {"mnemonic": "INC BC",           "code": "03",               "use": ""             },
    {"mnemonic": "INC DE",           "code": "13",               "use": ""             },
    {"mnemonic": "INC HL",           "code": "23",               "use": ""             },
    {"mnemonic": "INC SP",           "code": "33",               "use": ""             },
    {"mnemonic": "INC IX",           "code": "DD23",             "use": ""             },
    {"mnemonic": "INC IY",           "code": "FD23",             "use": ""             },
    {"mnemonic": "JP (HL)",          "code": "E9",               "use": ""             },
    {"mnemonic": "JP (IX)",          "code": "DDE9",             "use": ""             },
    {"mnemonic": "JP (IY)",          "code": "FDE9",             "use": ""             },
    {"mnemonic": "JP t",             "code": "C3t0t1",           "use": "t=code"       },
    {"mnemonic": "JP NZ,t",          "code": "C2t0t1",           "use": "t=code"       },
    {"mnemonic": "JP Z,t",           "code": "CAt0t1",           "use": "t=code"       },
    {"mnemonic": "JP NC,t",          "code": "D2t0t1",           "use": "t=code"       },
    {"mnemonic": "JP C,t",           "code": "DAt0t1",           "use": "t=code"       },
    {"mnemonic": "JP PO,t",          "code": "E2t0t1",           "use": "t=code"       },
    {"mnemonic": "JP PE,t",          "code": "EAt0t1",           "use": "t=code"       },
    {"mnemonic": "JP P,t",           "code": "F2t0t1",           "use": "t=code"       },
    {"mnemonic": "JP M,t",           "code": "FAt0t1",           "use": "t=code"       },
    {"mnemonic": "JR r",             "code": "18rr",             "use": "r=code_pcr"   },
    {"mnemonic": "JR NZ,r",          "code": "20rr",             "use": "r=code_pcr"   },
    {"mnemonic": "JR Z,r",           "code": "28rr",             "use": "r=code_pcr"   },
    {"mnemonic": "JR NC,r",          "code": "30rr",             "use": "r=code_pcr"   },
    {"mnemonic": "JR C,r",           "code": "38rr",             "use": "r=code_pcr"   },
    {"mnemonic": "LD A,A",           "code": "7F",               "use": ""             },
    {"mnemonic": "LD A,B",           "code": "78",               "use": ""             },
    {"mnemonic": "LD A,C",           "code": "79",               "use": ""             },
    {"mnemonic": "LD A,D",           "code": "7A",               "use": ""             },
    {"mnemonic": "LD A,E",           "code": "7B",               "use": ""             },
    {"mnemonic": "LD A,H",           "code": "7C",               "use": ""             },
    {"mnemonic": "LD A,L",           "code": "7D",               "use": ""             },
    {"mnemonic": "LD A,IXL",         "code": "DD7D",             "use": ""             },
    {"mnemonic": "LD IXH,A",         "code": "FD6F",             "use": ""             },
    {"mnemonic": "LD A,(HL)",        "code": "7E",               "use": ""             },
    {"mnemonic": "LD A,(BC)",        "code": "0A",               "use": ""             },
    {"mnemonic": "LD A,(DE)",        "code": "1A",               "use": ""             },
    {"mnemonic": "LD A,(IX+i)",      "code": "DD7Eii",           "use": "i=const"      },
    {"mnemonic": "LD A,(IY+i)",      "code": "FD7Eii",           "use": "i=const"      },
    {"mnemonic": "LD B,A",           "code": "47",               "use": ""             },
    {"mnemonic": "LD B,B",           "code": "40",               "use": ""             },
    {"mnemonic": "LD B,C",           "code": "41",               "use": ""             },
    {"mnemonic": "LD B,D",           "code": "42",               "use": ""             },
    {"mnemonic": "LD B,E",           "code": "43",               "use": ""             },
    {"mnemonic": "LD B,H",           "code": "44",               "use": ""             },
    {"mnemonic": "LD B,L",           "code": "45",               "use": ""             },
    {"mnemonic": "LD B,(HL)",        "code": "46",               "use": ""             },
    {"mnemonic": "LD B,(IX+i)",      "code": "DD46ii",           "use": "i=const"      },
    {"mnemonic": "LD B,(IY+i)",      "code": "FD46ii",           "use": "i=const"      },
    {"mnemonic": "LD C,A",           "code": "4F",               "use": ""             },
    {"mnemonic": "LD C,B",           "code": "48",               "use": ""             },
    {"mnemonic": "LD C,C",           "code": "49",               "use": ""             },
    {"mnemonic": "LD C,D",           "code": "4A",               "use": ""             },
    {"mnemonic": "LD C,E",           "code": "4B",               "use": ""             },
    {"mnemonic": "LD C,H",           "code": "4C",               "use": ""             },
    {"mnemonic": "LD C,L",           "code": "4D",               "use": ""             },
    {"mnemonic": "LD C,(HL)",        "code": "4E",               "use": ""             },
    {"mnemonic": "LD C,(IX+i)",      "code": "DD4Eii",           "use": "i=const"      },
    {"mnemonic": "LD C,(IY+i)",      "code": "FD4Eii",           "use": "i=const"      },
    {"mnemonic": "LD D,A",           "code": "57",               "use": ""             },
    {"mnemonic": "LD D,B",           "code": "50",               "use": ""             },
    {"mnemonic": "LD D,C",           "code": "51",               "use": ""             },
    {"mnemonic": "LD D,D",           "code": "52",               "use": ""             },
    {"mnemonic": "LD D,E",           "code": "53",               "use": ""             },
    {"mnemonic": "LD D,H",           "code": "54",               "use": ""             },
    {"mnemonic": "LD D,L",           "code": "55",               "use": ""             },
    {"mnemonic": "LD D,(HL)",        "code": "56",               "use": ""             },
    {"mnemonic": "LD D,(IX+i)",      "code": "DD56ii",           "use": "i=const"      },
    {"mnemonic": "LD D,(IY+i)",      "code": "FD56ii",           "use": "i=const"      },
    {"mnemonic": "LD E,A",           "code": "5F",               "use": ""             },
    {"mnemonic": "LD E,B",           "code": "58",               "use": ""             },
    {"mnemonic": "LD E,C",           "code": "59",               "use": ""             },
    {"mnemonic": "LD E,D",           "code": "5A",               "use": ""             },
    {"mnemonic": "LD E,E",           "code": "5B",               "use": ""             },
    {"mnemonic": "LD E,H",           "code": "5C",               "use": ""             },
    {"mnemonic": "LD E,L",           "code": "5D",               "use": ""             },
    {"mnemonic": "LD E,(HL)",        "code": "5E",               "use": ""             },
    {"mnemonic": "LD E,(IX+i)",      "code": "DD5Eii",           "use": "i=const"      },
    {"mnemonic": "LD E,(IY+i)",      "code": "FD5Eii",           "use": "i=const"      },
    {"mnemonic": "LD H,A",           "code": "67",               "use": ""             },
    {"mnemonic": "LD H,B",           "code": "60",               "use": ""             },
    {"mnemonic": "LD H,C",           "code": "61",               "use": ""             },
    {"mnemonic": "LD H,D",           "code": "62",               "use": ""             },
    {"mnemonic": "LD IYH,D",         "code": "FD62",             "use": ""             },
    {"mnemonic": "LD H,E",           "code": "63",               "use": ""             },
    {"mnemonic": "LD H,H",           "code": "64",               "use": ""             },
    {"mnemonic": "LD H,L",           "code": "65",               "use": ""             },
    {"mnemonic": "LD H,(HL)",        "code": "66",               "use": ""             },
    {"mnemonic": "LD H,(IX+i)",      "code": "DD66ii",           "use": "i=const"      },
    {"mnemonic": "LD H,(IY+i)",      "code": "FD66ii",           "use": "i=const"      },
    {"mnemonic": "LD L,A",           "code": "6F",               "use": ""             },
    {"mnemonic": "LD IXL,A",         "code": "DD6F",             "use": ""             },
    {"mnemonic": "LD L,B",           "code": "68",               "use": ""             },
    {"mnemonic": "LD L,C",           "code": "69",               "use": ""             },
    {"mnemonic": "LD L,D",           "code": "6A",               "use": ""             },
    {"mnemonic": "LD L,E",           "code": "6B",               "use": ""             },
    {"mnemonic": "LD L,H",           "code": "6C",               "use": ""             },
    {"mnemonic": "LD L,L",           "code": "6D",               "use": ""             },
    {"mnemonic": "LD L,(HL)",        "code": "6E",               "use": ""             },
    {"mnemonic": "LD L,(IX+i)",      "code": "DD6Eii",           "use": "i=const"      },
    {"mnemonic": "LD L,(IY+i)",      "code": "FD6Eii",           "use": "i=const"      },
    {"mnemonic": "LD (HL),A",        "code": "77",               "use": ""             },
    {"mnemonic": "LD (HL),B",        "code": "70",               "use": ""             },
    {"mnemonic": "LD (HL),C",        "code": "71",               "use": ""             },
    {"mnemonic": "LD (HL),D",        "code": "72",               "use": ""             },
    {"mnemonic": "LD (HL),E",        "code": "73",               "use": ""             },
    {"mnemonic": "LD (HL),H",        "code": "74",               "use": ""             },
    {"mnemonic": "LD (HL),L",        "code": "75",               "use": ""             },
    {"mnemonic": "LD (IX+i),A",      "code": "DD77ii",           "use": "i=const"      },
    {"mnemonic": "LD (IX+i),B",      "code": "DD70ii",           "use": "i=const"      },
    {"mnemonic": "LD (IX+i),C",      "code": "DD71ii",           "use": "i=const"      },
    {"mnemonic": "LD (IX+i),D",      "code": "DD72ii",           "use": "i=const"      },
    {"mnemonic": "LD (IX+i),E",      "code": "DD73ii",           "use": "i=const"      },
    {"mnemonic": "LD (IX+i),H",      "code": "DD74ii",           "use": "i=const"      },
    {"mnemonic": "LD (IX+i),L",      "code": "DD75ii",           "use": "i=const"      },
    {"mnemonic": "LD (IY+i),A",      "code": "FD77ii",           "use": "i=const"      },
    {"mnemonic": "LD (IY+i),B",      "code": "FD70ii",           "use": "i=const"      },
    {"mnemonic": "LD (IY+i),C",      "code": "FD71ii",           "use": "i=const"      },
    {"mnemonic": "LD (IY+i),D",      "code": "FD72ii",           "use": "i=const"      },
    {"mnemonic": "LD (IY+i),E",      "code": "FD73ii",           "use": "i=const"      },
    {"mnemonic": "LD (IY+i),H",      "code": "FD74ii",           "use": "i=const"      },
    {"mnemonic": "LD (IY+i),L",      "code": "FD75ii",           "use": "i=const"      },
    {"mnemonic": "LD (BC),A",        "code": "02",               "use": ""             },
    {"mnemonic": "LD (DE),A",        "code": "12",               "use": ""             },
    {"mnemonic": "LD SP,HL",         "code": "F9",               "use": ""             },
    {"mnemonic": "LD SP,IX",         "code": "DDF9",             "use": ""             },
    {"mnemonic": "LD SP,IY",         "code": "FDF9",             "use": ""             },
    {"mnemonic": "LD I,A",           "code": "ED47",             "use": ""             },
    {"mnemonic": "LD A,I",           "code": "ED57",             "use": ""             },
    {"mnemonic": "LD R,A",           "code": "ED4F",             "use": ""             },
    {"mnemonic": "LD A,R",           "code": "ED5F",             "use": ""             },
    {"mnemonic": "LD A,(t)",         "code": "3At0t1",           "use": "t=data_r"     },
    {"mnemonic": "LD HL,(t)",        "code": "2At0t1",           "use": "t=data_r"     },
    {"mnemonic": "LD BC,(t)",        "code": "ED4Bt0t1",         "use": "t=data_r"     },
    {"mnemonic": "LD DE,(t)",        "code": "ED5Bt0t1",         "use": "t=data_r"     },
    {"mnemonic": "LD HL,(t)",        "code": "ED6Bt0t1",         "use": "t=data_r"     },
    {"mnemonic": "LD SP,(t)",        "code": "ED7Bt0t1",         "use": "t=data_r"     },
    {"mnemonic": "LD IX,(t)",        "code": "DD2At0t1",         "use": "t=data_r"     },
    {"mnemonic": "LD IY,(t)",        "code": "FD2At0t1",         "use": "t=data_r"     },
    {"mnemonic": "LD (t),A",         "code": "32t0t1",           "use": "t=data_w"     },
    {"mnemonic": "LD (t),HL",        "code": "22t0t1",           "use": "t=data_w"     },
    {"mnemonic": "LD (t),BC",        "code": "ED43t0t1",         "use": "t=data_w"     },
    {"mnemonic": "LD (t),DE",        "code": "ED53t0t1",         "use": "t=data_w"     },
    {"mnemonic": "LD (t),IX",        "code": "DD22t0t1",         "use": "t=data_w"     },
    {"mnemonic": "LD (t),IY",        "code": "FD22t0t1",         "use": "t=data_w"     },
    {"mnemonic": "LD (t),SP",        "code": "ED73t0t1",         "use": "t=data_w"     },
    {"mnemonic": "LD (IX+i),b",      "code": "DD36iibb",         "use": "i=const,b=const" },
    {"mnemonic": "LD (IY+i),b",      "code": "FD36iibb",         "use": "i=const,b=const" },
    {"mnemonic": "LD BC,w",          "code": "01w0w1",           "use": "w=const"      },
    {"mnemonic": "LD DE,w",          "code": "11w0w1",           "use": "w=const"      },
    {"mnemonic": "LD HL,w",          "code": "21w0w1",           "use": "w=const"      },
    {"mnemonic": "LD SP,w",          "code": "31w0w1",           "use": "w=const"      },
    {"mnemonic": "LD IX,w",          "code": "DD21w0w1",         "use": "w=const"      },
    {"mnemonic": "LD IY,w",          "code": "FD21w0w1",         "use": "w=const"      },
    {"mnemonic": "LD A,b",           "code": "3Ebb",             "use": "b=const"      },
    {"mnemonic": "LD B,b",           "code": "06bb",             "use": "b=const"      },
    {"mnemonic": "LD C,b",           "code": "0Ebb",             "use": "b=const"      },
    {"mnemonic": "LD D,b",           "code": "16bb",             "use": "b=const"      },
    {"mnemonic": "LD E,b",           "code": "1Ebb",             "use": "b=const"      },
    {"mnemonic": "LD H,b",           "code": "26bb",             "use": "b=const"      },
    {"mnemonic": "LD L,b",           "code": "2Ebb",             "use": "b=const"      },
    {"mnemonic": "LD (HL),b",        "code": "36bb",             "use": "b=const"      },
    {"mnemonic": "LDD",              "code": "EDA8",             "use": ""             },
    {"mnemonic": "LDI",              "code": "EDA0",             "use": ""             },
    {"mnemonic": "NEG",              "code": "ED44",             "use": ""             },
    {"mnemonic": "NOP",              "code": "00",               "use": ""             },
    {"mnemonic": "OUT (C),A",        "code": "ED79",             "use": ""             },
    {"mnemonic": "OUT (C),B",        "code": "ED41",             "use": ""             },
    {"mnemonic": "OUT (C),C",        "code": "ED49",             "use": ""             },
    {"mnemonic": "OUT (C),D",        "code": "ED51",             "use": ""             },
    {"mnemonic": "OUT (C),E",        "code": "ED59",             "use": ""             },
    {"mnemonic": "OUT (C),H",        "code": "ED61",             "use": ""             },
    {"mnemonic": "OUT (C),L",        "code": "ED69",             "use": ""             },
    {"mnemonic": "OUT (o),A",        "code": "D3oo",             "use": "o=port_w"     },
    {"mnemonic": "RES 0,A",          "code": "CB87",             "use": ""             },
    {"mnemonic": "RES 0,B",          "code": "CB80",             "use": ""             },
    {"mnemonic": "RES 0,C",          "code": "CB81",             "use": ""             },
    {"mnemonic": "RES 0,D",          "code": "CB82",             "use": ""             },
    {"mnemonic": "RES 0,E",          "code": "CB83",             "use": ""             },
    {"mnemonic": "RES 0,H",          "code": "CB84",             "use": ""             },
    {"mnemonic": "RES 0,L",          "code": "CB85",             "use": ""             },
    {"mnemonic": "RES 1,A",          "code": "CB8F",             "use": ""             },
    {"mnemonic": "RES 1,B",          "code": "CB88",             "use": ""             },
    {"mnemonic": "RES 1,C",          "code": "CB89",             "use": ""             },
    {"mnemonic": "RES 1,D",          "code": "CB8A",             "use": ""             },
    {"mnemonic": "RES 1,E",          "code": "CB8B",             "use": ""             },
    {"mnemonic": "RES 1,H",          "code": "CB8C",             "use": ""             },
    {"mnemonic": "RES 1,L",          "code": "CB8D",             "use": ""             },
    {"mnemonic": "RES 2,A",          "code": "CB97",             "use": ""             },
    {"mnemonic": "RES 2,B",          "code": "CB90",             "use": ""             },
    {"mnemonic": "RES 2,C",          "code": "CB91",             "use": ""             },
    {"mnemonic": "RES 2,D",          "code": "CB92",             "use": ""             },
    {"mnemonic": "RES 2,E",          "code": "CB93",             "use": ""             },
    {"mnemonic": "RES 2,H",          "code": "CB94",             "use": ""             },
    {"mnemonic": "RES 2,L",          "code": "CB95",             "use": ""             },
    {"mnemonic": "RES 3,A",          "code": "CB9F",             "use": ""             },
    {"mnemonic": "RES 3,B",          "code": "CB98",             "use": ""             },
    {"mnemonic": "RES 3,C",          "code": "CB99",             "use": ""             },
    {"mnemonic": "RES 3,D",          "code": "CB9A",             "use": ""             },
    {"mnemonic": "RES 3,E",          "code": "CB9B",             "use": ""             },
    {"mnemonic": "RES 3,H",          "code": "CB9C",             "use": ""             },
    {"mnemonic": "RES 3,L",          "code": "CB9D",             "use": ""             },
    {"mnemonic": "RES 4,A",          "code": "CBA7",             "use": ""             },
    {"mnemonic": "RES 4,B",          "code": "CBA0",             "use": ""             },
    {"mnemonic": "RES 4,C",          "code": "CBA1",             "use": ""             },
    {"mnemonic": "RES 4,D",          "code": "CBA2",             "use": ""             },
    {"mnemonic": "RES 4,E",          "code": "CBA3",             "use": ""             },
    {"mnemonic": "RES 4,H",          "code": "CBA4",             "use": ""             },
    {"mnemonic": "RES 4,L",          "code": "CBA5",             "use": ""             },
    {"mnemonic": "RES 5,A",          "code": "CBAF",             "use": ""             },
    {"mnemonic": "RES 5,B",          "code": "CBA8",             "use": ""             },
    {"mnemonic": "RES 5,C",          "code": "CBA9",             "use": ""             },
    {"mnemonic": "RES 5,D",          "code": "CBAA",             "use": ""             },
    {"mnemonic": "RES 5,E",          "code": "CBAB",             "use": ""             },
    {"mnemonic": "RES 5,H",          "code": "CBAC",             "use": ""             },
    {"mnemonic": "RES 5,L",          "code": "CBAD",             "use": ""             },
    {"mnemonic": "RES 6,A",          "code": "CBB7",             "use": ""             },
    {"mnemonic": "RES 6,B",          "code": "CBB0",             "use": ""             },
    {"mnemonic": "RES 6,C",          "code": "CBB1",             "use": ""             },
    {"mnemonic": "RES 6,D",          "code": "CBB2",             "use": ""             },
    {"mnemonic": "RES 6,E",          "code": "CBB3",             "use": ""             },
    {"mnemonic": "RES 6,H",          "code": "CBB4",             "use": ""             },
    {"mnemonic": "RES 6,L",          "code": "CBB5",             "use": ""             },
    {"mnemonic": "RES 7,A",          "code": "CBBF",             "use": ""             },
    {"mnemonic": "RES 7,B",          "code": "CBB8",             "use": ""             },
    {"mnemonic": "RES 7,C",          "code": "CBB9",             "use": ""             },
    {"mnemonic": "RES 7,D",          "code": "CBBA",             "use": ""             },
    {"mnemonic": "RES 7,E",          "code": "CBBB",             "use": ""             },
    {"mnemonic": "RES 7,H",          "code": "CBBC",             "use": ""             },
    {"mnemonic": "RES 7,L",          "code": "CBBD",             "use": ""             },
    {"mnemonic": "RES 0,(HL)",       "code": "CB86",             "use": ""             },
    {"mnemonic": "RES 1,(HL)",       "code": "CB8E",             "use": ""             },
    {"mnemonic": "RES 2,(HL)",       "code": "CB96",             "use": ""             },
    {"mnemonic": "RES 3,(HL)",       "code": "CB9E",             "use": ""             },
    {"mnemonic": "RES 4,(HL)",       "code": "CBA6",             "use": ""             },
    {"mnemonic": "RES 5,(HL)",       "code": "CBAE",             "use": ""             },
    {"mnemonic": "RES 6,(HL)",       "code": "CBB6",             "use": ""             },
    {"mnemonic": "RES 7,(HL)",       "code": "CBBE",             "use": ""             },
    {"mnemonic": "RES 0,(IX+i)",     "code": "DDCBii86",         "use": "i=const"      },
    {"mnemonic": "RES 0,(IY+i)",     "code": "FDCBii86",         "use": "i=const"      },
    {"mnemonic": "RES 1,(IX+i)",     "code": "DDCBii8E",         "use": "i=const"      },
    {"mnemonic": "RES 1,(IY+i)",     "code": "FDCBii8E",         "use": "i=const"      },
    {"mnemonic": "RES 2,(IX+i)",     "code": "DDCBii96",         "use": "i=const"      },
    {"mnemonic": "RES 2,(IY+i)",     "code": "FDCBii96",         "use": "i=const"      },
    {"mnemonic": "RES 3,(IX+i)",     "code": "DDCBii9E",         "use": "i=const"      },
    {"mnemonic": "RES 3,(IY+i)",     "code": "FDCBii9E",         "use": "i=const"      },
    {"mnemonic": "RES 4,(IX+i)",     "code": "DDCBiiA6",         "use": "i=const"      },
    {"mnemonic": "RES 4,(IY+i)",     "code": "FDCBiiA6",         "use": "i=const"      },
    {"mnemonic": "RES 5,(IX+i)",     "code": "DDCBiiAE",         "use": "i=const"      },
    {"mnemonic": "RES 5,(IY+i)",     "code": "FDCBiiAE",         "use": "i=const"      },
    {"mnemonic": "RES 6,(IX+i)",     "code": "DDCBiiB6",         "use": "i=const"      },
    {"mnemonic": "RES 6,(IY+i)",     "code": "FDCBiiB6",         "use": "i=const"      },
    {"mnemonic": "RES 7,(IX+i)",     "code": "DDCBiiBE",         "use": "i=const"      },
    {"mnemonic": "RES 7,(IY+i)",     "code": "FDCBiiBE",         "use": "i=const"      },
    {"mnemonic": "SET 0,A",          "code": "CBC7",             "use": ""             },
    {"mnemonic": "SET 0,B",          "code": "CBC0",             "use": ""             },
    {"mnemonic": "SET 0,C",          "code": "CBC1",             "use": ""             },
    {"mnemonic": "SET 0,D",          "code": "CBC2",             "use": ""             },
    {"mnemonic": "SET 0,E",          "code": "CBC3",             "use": ""             },
    {"mnemonic": "SET 0,H",          "code": "CBC4",             "use": ""             },
    {"mnemonic": "SET 0,L",          "code": "CBC5",             "use": ""             },
    {"mnemonic": "SET 1,A",          "code": "CBCF",             "use": ""             },
    {"mnemonic": "SET 1,B",          "code": "CBC8",             "use": ""             },
    {"mnemonic": "SET 1,C",          "code": "CBC9",             "use": ""             },
    {"mnemonic": "SET 1,D",          "code": "CBCA",             "use": ""             },
    {"mnemonic": "SET 1,E",          "code": "CBCB",             "use": ""             },
    {"mnemonic": "SET 1,H",          "code": "CBCC",             "use": ""             },
    {"mnemonic": "SET 1,L",          "code": "CBCD",             "use": ""             },
    {"mnemonic": "SET 2,A",          "code": "CBD7",             "use": ""             },
    {"mnemonic": "SET 2,B",          "code": "CBD0",             "use": ""             },
    {"mnemonic": "SET 2,C",          "code": "CBD1",             "use": ""             },
    {"mnemonic": "SET 2,D",          "code": "CBD2",             "use": ""             },
    {"mnemonic": "SET 2,E",          "code": "CBD3",             "use": ""             },
    {"mnemonic": "SET 2,H",          "code": "CBD4",             "use": ""             },
    {"mnemonic": "SET 2,L",          "code": "CBD5",             "use": ""             },
    {"mnemonic": "SET 3,A",          "code": "CBDF",             "use": ""             },
    {"mnemonic": "SET 3,B",          "code": "CBD8",             "use": ""             },
    {"mnemonic": "SET 3,C",          "code": "CBD9",             "use": ""             },
    {"mnemonic": "SET 3,D",          "code": "CBDA",             "use": ""             },
    {"mnemonic": "SET 3,E",          "code": "CBDB",             "use": ""             },
    {"mnemonic": "SET 3,H",          "code": "CBDC",             "use": ""             },
    {"mnemonic": "SET 3,L",          "code": "CBDD",             "use": ""             },
    {"mnemonic": "SET 4,A",          "code": "CBE7",             "use": ""             },
    {"mnemonic": "SET 4,B",          "code": "CBE0",             "use": ""             },
    {"mnemonic": "SET 4,C",          "code": "CBE1",             "use": ""             },
    {"mnemonic": "SET 4,D",          "code": "CBE2",             "use": ""             },
    {"mnemonic": "SET 4,E",          "code": "CBE3",             "use": ""             },
    {"mnemonic": "SET 4,H",          "code": "CBE4",             "use": ""             },
    {"mnemonic": "SET 4,L",          "code": "CBE5",             "use": ""             },
    {"mnemonic": "SET 5,A",          "code": "CBEF",             "use": ""             },
    {"mnemonic": "SET 5,B",          "code": "CBE8",             "use": ""             },
    {"mnemonic": "SET 5,C",          "code": "CBE9",             "use": ""             },
    {"mnemonic": "SET 5,D",          "code": "CBEA",             "use": ""             },
    {"mnemonic": "SET 5,E",          "code": "CBEB",             "use": ""             },
    {"mnemonic": "SET 5,H",          "code": "CBEC",             "use": ""             },
    {"mnemonic": "SET 5,L",          "code": "CBED",             "use": ""             },
    {"mnemonic": "SET 6,A",          "code": "CBF7",             "use": ""             },
    {"mnemonic": "SET 6,B",          "code": "CBF0",             "use": ""             },
    {"mnemonic": "SET 6,C",          "code": "CBF1",             "use": ""             },
    {"mnemonic": "SET 6,D",          "code": "CBF2",             "use": ""             },
    {"mnemonic": "SET 6,E",          "code": "CBF3",             "use": ""             },
    {"mnemonic": "SET 6,H",          "code": "CBF4",             "use": ""             },
    {"mnemonic": "SET 6,L",          "code": "CBF5",             "use": ""             },
    {"mnemonic": "SET 7,A",          "code": "CBFF",             "use": ""             },
    {"mnemonic": "SET 7,B",          "code": "CBF8",             "use": ""             },
    {"mnemonic": "SET 7,C",          "code": "CBF9",             "use": ""             },
    {"mnemonic": "SET 7,D",          "code": "CBFA",             "use": ""             },
    {"mnemonic": "SET 7,E",          "code": "CBFB",             "use": ""             },
    {"mnemonic": "SET 7,H",          "code": "CBFC",             "use": ""             },
    {"mnemonic": "SET 7,L",          "code": "CBFD",             "use": ""             },
    {"mnemonic": "SET 0,(HL)",       "code": "CBC6",             "use": ""             },
    {"mnemonic": "SET 1,(HL)",       "code": "CBCE",             "use": ""             },
    {"mnemonic": "SET 2,(HL)",       "code": "CBD6",             "use": ""             },
    {"mnemonic": "SET 3,(HL)",       "code": "CBDE",             "use": ""             },
    {"mnemonic": "SET 4,(HL)",       "code": "CBE6",             "use": ""             },
    {"mnemonic": "SET 5,(HL)",       "code": "CBEE",             "use": ""             },
    {"mnemonic": "SET 6,(HL)",       "code": "CBF6",             "use": ""             },
    {"mnemonic": "SET 7,(HL)",       "code": "CBFE",             "use": ""             },
    {"mnemonic": "SET 0,(IX+i)",     "code": "DDCBiiC6",         "use": "i=const"      },
    {"mnemonic": "SET 0,(IY+i)",     "code": "FDCBiiC6",         "use": "i=const"      },
    {"mnemonic": "SET 1,(IX+i)",     "code": "DDCBiiCE",         "use": "i=const"      },
    {"mnemonic": "SET 1,(IY+i)",     "code": "FDCBiiCE",         "use": "i=const"      },
    {"mnemonic": "SET 2,(IX+i)",     "code": "DDCBiiD6",         "use": "i=const"      },
    {"mnemonic": "SET 2,(IY+i)",     "code": "FDCBiiD6",         "use": "i=const"      },
    {"mnemonic": "SET 3,(IX+i)",     "code": "DDCBiiDE",         "use": "i=const"      },
    {"mnemonic": "SET 3,(IY+i)",     "code": "FDCBiiDE",         "use": "i=const"      },
    {"mnemonic": "SET 4,(IX+i)",     "code": "DDCBiiE6",         "use": "i=const"      },
    {"mnemonic": "SET 4,(IY+i)",     "code": "FDCBiiE6",         "use": "i=const"      },
    {"mnemonic": "SET 5,(IX+i)",     "code": "DDCBiiEE",         "use": "i=const"      },
    {"mnemonic": "SET 5,(IY+i)",     "code": "FDCBiiEE",         "use": "i=const"      },
    {"mnemonic": "SET 6,(IX+i)",     "code": "DDCBiiF6",         "use": "i=const"      },
    {"mnemonic": "SET 6,(IY+i)",     "code": "FDCBiiF6",         "use": "i=const"      },
    {"mnemonic": "SET 7,(IX+i)",     "code": "DDCBiiFE",         "use": "i=const"      },
    {"mnemonic": "SET 7,(IY+i)",     "code": "FDCBiiFE",         "use": "i=const"      },
    {"mnemonic": "RET",              "code": "C9",               "use": ""             },
    {"mnemonic": "POP BC",           "code": "C1",               "use": ""             },
    {"mnemonic": "POP DE",           "code": "D1",               "use": ""             },
    {"mnemonic": "POP HL",           "code": "E1",               "use": ""             },
    {"mnemonic": "POP AF",           "code": "F1",               "use": ""             },
    {"mnemonic": "POP IX",           "code": "DDE1",             "use": ""             },
    {"mnemonic": "POP IY",           "code": "FDE1",             "use": ""             },
    {"mnemonic": "PUSH BC",          "code": "C5",               "use": ""             },
    {"mnemonic": "PUSH DE",          "code": "D5",               "use": ""             },
    {"mnemonic": "PUSH HL",          "code": "E5",               "use": ""             },
    {"mnemonic": "PUSH AF",          "code": "F5",               "use": ""             },
    {"mnemonic": "PUSH IX",          "code": "DDE5",             "use": ""             },
    {"mnemonic": "PUSH IY",          "code": "FDE5",             "use": ""             },
    {"mnemonic": "RET NZ",           "code": "C0",               "use": ""             },
    {"mnemonic": "RET Z",            "code": "C8",               "use": ""             },
    {"mnemonic": "RET NC",           "code": "D0",               "use": ""             },
    {"mnemonic": "RET C",            "code": "D8",               "use": ""             },
    {"mnemonic": "RET PO",           "code": "E0",               "use": ""             },
    {"mnemonic": "RET PE",           "code": "E8",               "use": ""             },
    {"mnemonic": "RET P",            "code": "F0",               "use": ""             },
    {"mnemonic": "RET M",            "code": "F8",               "use": ""             },
    {"mnemonic": "RL A",             "code": "CB17",             "use": ""             },
    {"mnemonic": "RL B",             "code": "CB10",             "use": ""             },
    {"mnemonic": "RL C",             "code": "CB11",             "use": ""             },
    {"mnemonic": "RL D",             "code": "CB12",             "use": ""             },
    {"mnemonic": "RL E",             "code": "CB13",             "use": ""             },
    {"mnemonic": "RL H",             "code": "CB14",             "use": ""             },
    {"mnemonic": "RL L",             "code": "CB15",             "use": ""             },
    {"mnemonic": "RL (HL)",          "code": "CB16",             "use": ""             },
    {"mnemonic": "RL (IX+i)",        "code": "DDCBii16",         "use": "i=const"      },
    {"mnemonic": "RL (IY+i)",        "code": "FDCBii16",         "use": "i=const"      },
    {"mnemonic": "RLA",              "code": "17",               "use": ""             },
    {"mnemonic": "RLC A",            "code": "CB07",             "use": ""             },
    {"mnemonic": "RLC B",            "code": "CB00",             "use": ""             },
    {"mnemonic": "RLC C",            "code": "CB01",             "use": ""             },
    {"mnemonic": "RLC D",            "code": "CB02",             "use": ""             },
    {"mnemonic": "RLC E",            "code": "CB03",             "use": ""             },
    {"mnemonic": "RLC H",            "code": "CB04",             "use": ""             },
    {"mnemonic": "RLC L",            "code": "CB05",             "use": ""             },
    {"mnemonic": "RLC (HL)",         "code": "CB06",             "use": ""             },
    {"mnemonic": "RLC (IX+i)",       "code": "DDCBii06",         "use": "i=const"      },
    {"mnemonic": "RLC (IY+i)",       "code": "FDCBii06",         "use": "i=const"      },
    {"mnemonic": "RLCA",             "code": "07",               "use": ""             },
    {"mnemonic": "RR A",             "code": "CB1F",             "use": ""             },
    {"mnemonic": "RR B",             "code": "CB18",             "use": ""             },
    {"mnemonic": "RR C",             "code": "CB19",             "use": ""             },
    {"mnemonic": "RR D",             "code": "CB1A",             "use": ""             },
    {"mnemonic": "RR E",             "code": "CB1B",             "use": ""             },
    {"mnemonic": "RR H",             "code": "CB1C",             "use": ""             },
    {"mnemonic": "RR L",             "code": "CB1D",             "use": ""             },
    {"mnemonic": "RR (HL)",          "code": "CB1E",             "use": ""             },
    {"mnemonic": "RR (IX+i)",        "code": "DDCBii1E",         "use": "i=const"      },
    {"mnemonic": "RR (IY+i)",        "code": "FDCBii1E",         "use": "i=const"      },
    {"mnemonic": "RRA",              "code": "1F",               "use": ""             },
    {"mnemonic": "RRC A",            "code": "CB0F",             "use": ""             },
    {"mnemonic": "RRC B",            "code": "CB08",             "use": ""             },
    {"mnemonic": "RRC C",            "code": "CB09",             "use": ""             },
    {"mnemonic": "RRC D",            "code": "CB0A",             "use": ""             },
    {"mnemonic": "RRC E",            "code": "CB0B",             "use": ""             },
    {"mnemonic": "RRC H",            "code": "CB0C",             "use": ""             },
    {"mnemonic": "RRC L",            "code": "CB0D",             "use": ""             },
    {"mnemonic": "RRC (HL)",         "code": "CB0E",             "use": ""             },
    {"mnemonic": "RRC (IX+i)",       "code": "DDCBii0E",         "use": "i=const"      },
    {"mnemonic": "RRC (IY+i)",       "code": "FDCBii0E",         "use": "i=const"      },
    {"mnemonic": "RRCA",             "code": "0F",               "use": ""             },
    {"mnemonic": "SLA A",            "code": "CB27",             "use": ""             },
    {"mnemonic": "SLA B",            "code": "CB20",             "use": ""             },
    {"mnemonic": "SLA C",            "code": "CB21",             "use": ""             },
    {"mnemonic": "SLA D",            "code": "CB22",             "use": ""             },
    {"mnemonic": "SLA E",            "code": "CB23",             "use": ""             },
    {"mnemonic": "SLA H",            "code": "CB24",             "use": ""             },
    {"mnemonic": "SLA L",            "code": "CB25",             "use": ""             },
    {"mnemonic": "SLA (HL)",         "code": "CB26",             "use": ""             },
    {"mnemonic": "SLA (IX+i)",       "code": "DDCBii26",         "use": "i=const"      },
    {"mnemonic": "SLA (IY+i)",       "code": "FDCBii26",         "use": "i=const"      },
    {"mnemonic": "SRA A",            "code": "CB2F",             "use": ""             },
    {"mnemonic": "SRA B",            "code": "CB28",             "use": ""             },
    {"mnemonic": "SRA C",            "code": "CB29",             "use": ""             },
    {"mnemonic": "SRA D",            "code": "CB2A",             "use": ""             },
    {"mnemonic": "SRA E",            "code": "CB2B",             "use": ""             },
    {"mnemonic": "SRA H",            "code": "CB2C",             "use": ""             },
    {"mnemonic": "SRA L",            "code": "CB2D",             "use": ""             },
    {"mnemonic": "SRA (HL)",         "code": "CB2E",             "use": ""             },
    {"mnemonic": "SRA (IX+i)",       "code": "DDCBii2E",         "use": "i=const"      },
    {"mnemonic": "SRA (IY+i)",       "code": "FDCBii2E",         "use": "i=const"      },
    {"mnemonic": "SRL A",            "code": "CB3F",             "use": ""             },
    {"mnemonic": "SRL B",            "code": "CB38",             "use": ""             },
    {"mnemonic": "SRL C",            "code": "CB39",             "use": ""             },
    {"mnemonic": "SRL D",            "code": "CB3A",             "use": ""             },
    {"mnemonic": "SRL E",            "code": "CB3B",             "use": ""             },
    {"mnemonic": "SRL H",            "code": "CB3C",             "use": ""             },
    {"mnemonic": "SRL L",            "code": "CB3D",             "use": ""             },
    {"mnemonic": "SRL (HL)",         "code": "CB3E",             "use": ""             },
    {"mnemonic": "SRL (IX+i)",       "code": "DDCBii3E",         "use": "i=const"      },
    {"mnemonic": "SRL (IY+i)",       "code": "FDCBii3E",         "use": "i=const"      },
    {"mnemonic": "RST 0X00",         "code": "C7",               "use": ""             },
    {"mnemonic": "RST 0X08",         "code": "CF",               "use": ""             },
    {"mnemonic": "RST 0X10",         "code": "D7",               "use": ""             },
    {"mnemonic": "RST 0X18",         "code": "DF",               "use": ""             },
    {"mnemonic": "RST 0X20",         "code": "E7",               "use": ""             },
    {"mnemonic": "RST 0X28",         "code": "EF",               "use": ""             },
    {"mnemonic": "RST 0X30",         "code": "F7",               "use": ""             },
    {"mnemonic": "RST 0X38",         "code": "FF",               "use": ""             },
    {"mnemonic": "SCF",              "code": "37",               "use": ""             },
    {"mnemonic": "SBC A",            "code": "9F",               "use": ""             },
    {"mnemonic": "SBC B",            "code": "98",               "use": ""             },
    {"mnemonic": "SBC C",            "code": "99",               "use": ""             },
    {"mnemonic": "SBC D",            "code": "9A",               "use": ""             },
    {"mnemonic": "SBC E",            "code": "9B",               "use": ""             },
    {"mnemonic": "SBC H",            "code": "9C",               "use": ""             },
    {"mnemonic": "SBC L",            "code": "9D",               "use": ""             },
    {"mnemonic": "SBC (HL)",         "code": "9E",               "use": ""             },
    {"mnemonic": "SBC A,(IX+i)",     "code": "DD9Eii",           "use": "i=const"      },
    {"mnemonic": "SBC A,(IY+i)",     "code": "FD9Eii",           "use": "i=const"      },
    {"mnemonic": "SBC HL,BC",        "code": "ED42",             "use": ""             },
    {"mnemonic": "SBC HL,DE",        "code": "ED52",             "use": ""             },
    {"mnemonic": "SBC HL,HL",        "code": "ED62",             "use": ""             },
    {"mnemonic": "SBC HL,SP",        "code": "ED72",             "use": ""             },
    {"mnemonic": "SBC b",            "code": "DEbb",             "use": "b=const"      },
    {"mnemonic": "SUB A",            "code": "97",               "use": ""             },
    {"mnemonic": "SUB B",            "code": "90",               "use": ""             },
    {"mnemonic": "SUB C",            "code": "91",               "use": ""             },
    {"mnemonic": "SUB D",            "code": "92",               "use": ""             },
    {"mnemonic": "SUB E",            "code": "93",               "use": ""             },
    {"mnemonic": "SUB H",            "code": "94",               "use": ""             },
    {"mnemonic": "SUB L",            "code": "95",               "use": ""             },
    {"mnemonic": "SUB (HL)",         "code": "96",               "use": ""             },
    {"mnemonic": "SUB (IX+i)",       "code": "DD96ii",           "use": "i=const"      },
    {"mnemonic": "SUB (IY+i)",       "code": "FD96ii",           "use": "i=const"      },
    {"mnemonic": "SUB b",            "code": "D6bb",             "use": "b=const"      },
    {"mnemonic": "LD A,(BC)",        "code": "DD0A",             "use": ""             },
    {"mnemonic": "DEC DE",           "code": "DD1B",             "use": ""             },
    {"mnemonic": "LD IXH,b",         "code": "DD26bb",           "use": "b=const"      },
    {"mnemonic": "INC IXL",          "code": "DD2C",             "use": ""             },
    {"mnemonic": "DEC IXL",          "code": "DD2D",             "use": ""             },
    {"mnemonic": "LD IXL,b",         "code": "DD2Ebb",           "use": "b=const"      },
    {"mnemonic": "LD C,IXH",         "code": "DD4C",             "use": ""             },
    {"mnemonic": "LD D,IXH",         "code": "DD54",             "use": ""             },
    {"mnemonic": "LD E,IXL",         "code": "DD5D",             "use": ""             },
    {"mnemonic": "LD IXH,B",         "code": "DD60",             "use": ""             },
    {"mnemonic": "LD IXH,C",         "code": "DD61",             "use": ""             },
    {"mnemonic": "LD IXH,D",         "code": "DD62",             "use": ""             },
    {"mnemonic": "LD IXH,E",         "code": "DD63",             "use": ""             },
    {"mnemonic": "LD IXH,A",         "code": "DD67",             "use": ""             },
    {"mnemonic": "LD A,IXH",         "code": "DD7C",             "use": ""             },
    {"mnemonic": "ADD A,IXH",        "code": "DD84",             "use": ""             },
    {"mnemonic": "SUB IXH",          "code": "DD94",             "use": ""             },
    {"mnemonic": "SUB IXL",          "code": "DD95",             "use": ""             },
    {"mnemonic": "XOR IXH",          "code": "DDAC",             "use": ""             },
    {"mnemonic": "DEC BC",           "code": "FD0B",             "use": ""             },
    {"mnemonic": "DEC DE",           "code": "FD1B",             "use": ""             },
    {"mnemonic": "EI",               "code": "FDFB",             "use": ""             },
    {"mnemonic": "LD IYL,E",         "code": "FD6B",             "use": ""             },
    {"mnemonic": "LD A,IYH",         "code": "FD7C",             "use": ""             },
    {"mnemonic": "LD A,IYL",         "code": "FD7D",             "use": ""             },
    {"mnemonic": "JR NZ,r",          "code": "FD20rr",           "use": "r=code_pcr"   },
    {"mnemonic": "DEC IYH",          "code": "FD25",             "use": ""             },
    {"mnemonic": "LD C,B",           "code": "FD48",             "use": ""             },
    {"mnemonic": "LD IYH,E",         "code": "FD63",             "use": ""             },
    {"mnemonic": "XOR IYH",          "code": "FDAC",             "use": ""             },
    {"mnemonic": "CALL Z,t",         "code": "FDCCt0t1",         "use": "t=code"       },
    {"mnemonic": "RET C",            "code": "FDD8",             "use": ""             },
]


class CPU_Z80(opcodetools.cpu.base_cpu.CPU):

    def __init__(self):
        super().__init__(OPCODES)



import os

import opcodetools.cpu.cpu_manager
from typing import List


class ASMException(Exception):
    def __init__(self, message, line):
        super().__init__(message)
        self.line = line


class Assembler:

    '''Manages labels and controls the assembly process
    '''

    def __init__(self, filename: str):
        '''Create a new Assembler object

        Args:
            filename (string) name of file to assemble
        '''

        self.lines = self.load_lines(filename)
        self.code = self.remove_comments_and_blanks(self.lines)
        self.defines = {}
        self.labels = {}
        self.cpu = None

    def load_lines(self, filename: str) -> List[dict]:
        '''Load the lines from the given file

        This method also recurses into included files.

        The information about a line looks like is:
        {
            'file_name': 'test.asm',
            'line_number' : 4
            'text' : '   MOV A,#$50'
        }

        Args:
            filename (str): the name of the file

        Returns:
            list : list of line information from the file (and includes)
        '''

        absn = os.path.abspath(filename)
        basep = os.path.dirname(absn)

        with open(filename, 'r') as f:
            raw_lines = f.readlines()

        ret = []
        pos = 0
        for line in raw_lines:
            pos += 1
            n = line.strip()
            if n.startswith('.include'):
                # TODO error checking/reporting
                n = n[8:].strip()
                n = os.path.join(basep, n)
                try:
                    ret = ret + self.load_lines(n)
                except FileNotFoundError:
                    raise ASMException('Could not find file ' + n, {'file_name': n, 'line_number': pos, 'text': line})
                continue
            ret.append({
                'file_name': filename,
                'line_number': pos,
                'text': n
            })
        return ret

    def remove_comments_and_blanks(self, lines):
        '''Make a list of code lines (no blanks, no comments)

        For each line returned, the "original_text" has the unaltered

        Args:
            lines (list): complete list of lines from the files

        Returns:
            List : just the lines of code (and without comments)
        '''
        ret = []
        for line in lines:
            n = line['text']
            i = n.find(';')
            if i >= 0:
                n = n[0:i].strip()
            if n:
                line['original_text'] = line['text']
                line['text'] = n
                ret.append(line)
        return ret

    def process_data_term(self, line, pass_number: int, cur_term: str):
        '''Process a numerical value

        Args:
            line: the code line
            pass_number: 0 or 1
            cur_term: the term to parse
        Returns:
            List: a list of bytes (first-pass return all 0s)

        '''
        is_word = False
        if(cur_term.startswith('word ')):
            is_word = True
            cur_term = cur_term[5:].strip()
        elif(cur_term.startswith('byte ')):
            is_word = False
            cur_term = cur_term[5:].strip()

        if cur_term[0] >= '0' and cur_term[0] <= '9':
            cur_term = cur_term.replace('_', '')
            cur_term = cur_term.replace('.', '0')

        if pass_number == 0:
            if is_word:
                return [0, 0]
            else:
                return [0]
        else:
            try:
                if is_word:
                    ret = self.parse_numeric(cur_term)
                    if ret > 65535:
                        raise ASMException('Value is larger than two bytes', line)
                    return self.cpu.make_word(ret)
                else:
                    ret = self.parse_numeric(cur_term)
                    if ret > 255:
                        raise ASMException('Value is larger than one byte', line)
                    return [ret]
            except ASMException:
                raise
            except Exception:
                raise ASMException('Invalid numeric value: ' + cur_term, line)

    def process_directive_data(self, line, pass_number: int):
        '''Process a data directive

        Creates the data list for a data-directive command

        Args:
            line : the code line
            pass_number : 0 or 1
        '''

        line['data'] = []

        cur_term = ''
        in_string = False

        for c in line['text'][1:]:
            if in_string:
                if c == '"':
                    in_string = False
                    for t in cur_term:
                        line['data'].append(ord(t))
                    cur_term = ''
                else:
                    cur_term = cur_term + c
            else:
                if c == '"':
                    in_string = True
                    cur_term = ''
                elif c == ',':
                    cur_term = cur_term.strip()
                    if cur_term:
                        dtt = self.process_data_term(line, pass_number, cur_term)
                        for d in dtt:
                            line['data'].append(d)
                    cur_term = ''
                else:
                    cur_term = cur_term + c

        if in_string:
            raise ASMException('Missing closing quotes', line)

        cur_term = cur_term.strip()
        if cur_term:
            dtt = self.process_data_term(line, pass_number, cur_term)
            for d in dtt:
                line['data'].append(d)

    def parse_numeric(self, s: str):
        '''Parse a numeric expression

        Uses the python eval to evalutate expressions,

        Args:
            s (str): the expression

        Returns:
            The evaluation value
        '''
        if s[0] == '<' or s[0] == '>':
            s = s[1:]
        z = {**self.labels, **self.defines}
        v = eval(s, None, z)
        return v

    def process_define(self, line, pass_number: int):
        '''Process a define

        Defines are of the form .VAR = VALUE. This method adds to the
        growing list of defines.

        Args:
            line : the code line
            pass_number (int): 0 or 1

        '''

        n = line['text']
        # print('##',n)
        i = n.index('=')
        v = n[i + 1:].strip()
        n = n[1:i].strip()
        if pass_number == 2 and (n in self.labels or n in self.defines):
            # Second pass ... handle multiply-defined errors
            raise ASMException('Multiply defined: ' + n, line)
        if n.startswith('_'):
            # Handle configs
            self.process_config_define(line, n, v)
        else:
            # Must be a numeric expression
            try:
                v = self.parse_numeric(v)
                self.defines[n] = v
            except Exception:
                raise ASMException('Invalid numeric constant: ' + v, line)

    def process_config_define(self, line: dict, key: str, value: str):
        '''Process config defines

        Config defines are of the form ._VAR = VALUE.
        '''
        if key == '_CPU':
            self.cpu = opcodetools.cpu.cpu_manager.get_cpu_by_name(value)
            if not self.cpu:
                raise ASMException('Unknown CPU: ' + value, line)
            self.cpu.init_assembly()
            if not self.cpu._opcodes[0].frags:
                self.cpu.make_frags()
        self.defines[key] = value

    def assemble(self):
        '''Two-pass assembly
        '''

        for pass_number in range(2):

            address = 0

            for line in self.code:

                n = line['text']

                if n.startswith('.'):

                    # Define (key = value)
                    i = n.find('"')  # In case the right side is a string, which might have '=' in it.
                    if i < 0:
                        i = len(n)
                    j = n.find('=')
                    if j < i and j >= 0:
                        self.process_define(line, pass_number)

                    # Data (list of bytes/words)
                    elif n.startswith('. ') or n.startswith('.word ') or n.startswith('.byte '):
                        self.process_directive_data(line, pass_number)
                        line['address'] = address

                    else:
                        raise ASMException('Unknown directive: ' + n, line)

                elif n.endswith(':'):

                    # Label (or origin)
                    n = n[:-1].strip()
                    if pass_number == 0:
                        if n in self.labels or n in self.defines:
                            raise ASMException(
                                'Multiply defined: ' + n, line)

                    try:
                        # A number ... this is an origin
                        if n.upper().startswith('0X'):
                            address = int(n[2:], 16)
                        else:
                            address = int(n)
                    except Exception:
                        # Not a number ... this is a label to remember
                        self.labels[n] = address

                else:

                    # Line of assembly
                    line['address'] = address
                    if not self.cpu:
                        raise ASMException('No CPU defined', line)

                    # Opcode
                    op = self.cpu.find_opcode_for_text(n, self)
                    if not op:
                        raise ASMException('Unknown opcode: ' + n, line)

                    try:
                        line['data'] = self.cpu.fill_in_opcode(n, self, address, op, pass_number)
                    except Exception as f:
                        raise ASMException(str(f), line)

                if 'data' in line:
                    address = address + len(line['data'])

    def write_listing(self, fname):
        '''Write the listing file

        Args:
            fname : the filename to create
        '''
        with open(fname, 'w') as f:
            f.write('#### Labels\n')
            keys = self.labels.keys()
            keys = sorted(keys)
            for label in keys:
                f.write('{:16} = 0x{:04X}\n'.format(label, self.labels[label]))
            f.write('\n')
            f.write('#### Defines\n')
            keys = self.defines.keys()
            keys = sorted(keys)
            for define in keys:
                v = self.defines[define]
                if isinstance(v, str):
                    f.write('{:16} = {}\n'.format(define, v))
                else:
                    f.write('{:16} = 0x{:04X}\n'.format(define, v))
            f.write('\n')

            for line in self.lines:
                addr = ''
                if 'address' in line:
                    addr = '{:04X}:'.format(line['address'])
                data = ''
                if 'data' in line:
                    for d in line['data']:
                        data = data + '{:02X} '.format(d)
                data = data.strip()
                txt = line['text']
                if 'original_text' in line:
                    txt = line['original_text']
                f.write('{} {:16} {}\n'.format(addr, data, txt))

    def write_binary(self, name):
        '''Write the binary file

        Args:
            fname : the filename to create
        '''
        for line in self.lines:
            if 'address' in line:
                org = line['address']
                break
        with open(name, 'wb') as f:
            for line in self.lines:
                if 'data' in line and line['data']:
                    new_org = line['address']
                    if new_org < org:
                        raise Exception('Origin problems')
                    while org < new_org:
                        f.write(bytearray([0xFF]))
                        org = org + 1
                    f.write(bytearray(line['data']))
                    org = org + len(bytearray(line['data']))




class AssemblyException(Exception):
    pass


class BaseAssembly:
    '''Baseclass of things a CPU needs to implement for assembly'''

    def init_assembly(self):
        '''Initialize the CPU to begin assembly

        This might be expensive. Don't do it until we know we need to.
        '''

        self.make_frags()

    def make_frags(self):
        '''Make opcode fragments for assembly

        The assembly processes needs the mnemonic broken up into fragments for matching.

        This method fills out the fragments for all opcodes. We assume that single lower-case
        letters represent things needing filled in.
        '''

        for op in self._opcodes:
            txt = op.mnemonic
            txt = self.remove_unneeded_whitespace(txt)
            op.frags = ['']
            for i in range(len(txt)):
                c = txt[i]
                if c.islower():
                    op.frags.append(c)
                    if i < (len(txt) - 1):
                        # More to go -- start a new string for the more
                        op.frags.append('')
                else:
                    op.frags[-1] = op.frags[-1] + c

    def remove_unneeded_whitespace(self, text: str):
        '''Remove unneeded whitespace from a string

        ASSUMPTION: All opcodes are one or two terms
            LDA (p,X)
        There are at most TWO terms separated by whitespace. All other whitespace
        can be removed.

        Args:
            text (str): the string

        Returns:
            str: the processed string
        '''
        i = text.find(' ')
        if i >= 0:
            ret = text[0:i + 1] + text[i + 1:].replace(' ', '')
        else:
            ret = text.replace(' ', '')

        return ret.strip()

    NON_SUB_CHARS = ',@#$~!?[]{}|'  # Add as needed

    def fix_up_special_opcodes(self, nmatch):
        return nmatch

    def find_opcode_for_text(self, text: str, assembler):
        '''Find the one opcode that best matches this line of text

        In finding the correct opcode, we parse out all the substitution
        pieces. That information is returned too.

        Args:
            text (str): the line of code
            assembler (Assembler): contains any defines

        Returns:
            Opcode: the requested (opcode,info) (or None)
        '''

        # Ignorable whitespace
        nmatch = self.remove_unneeded_whitespace(text)

        nmatch = self.fix_up_special_opcodes(nmatch)

        possibles = []
        possibles_info = []
        longest_first_frag = 0
        for op in self._opcodes:
            remain = nmatch
            info = {}
            for fi in range(len(op.frags)):
                if not remain:
                    # We've reached the end of the test
                    # '' means match, None means no match
                    break
                # Next fragment of the opcode being tested
                frag = op.frags[fi]
                if not frag[0].islower():
                    if remain.startswith(frag):
                        # This is a static fragment ... just peel it off
                        remain = remain[len(frag):]
                    else:
                        # This is a static fragment that doesn't match
                        remain = None  # This opcode can't match
                else:
                    # This is a substitution
                    if fi == len(op.frags) - 1:
                        # This is the last fragment ... match the rest of the line
                        i = len(remain)
                    else:
                        # Find the next fragment
                        i = remain.find(op.frags[fi + 1])
                    if i >= 0:
                        term = remain[:i]
                        info[frag[0]] = term
                        remain = remain[i:]
                        for c in self.NON_SUB_CHARS:
                            # These characters can't be in substitution parameters
                            if c in term:
                                remain = None
                                break
                    else:
                        remain = None

            if remain == '':
                # This opcode is a potential ... add it to the list
                possibles.append(op)
                possibles_info.append(info)
                if len(op.frags[0]) > longest_first_frag:
                    longest_first_frag = len(op.frags[0])

        # Rule out all but the longest first fragment
        for i in range(len(possibles) - 1, -1, -1):
            if len(possibles[i].frags[0]) != longest_first_frag:
                del possibles[i]
                del possibles_info[i]

        longest_frag = 0
        for p in possibles:
            if len(p.frags) > longest_frag:
                longest_frag = len(p.frags)

        # Rule out all but the most exact matches
        for i in range(len(possibles) - 1, -1, -1):
            if len(possibles[i].frags) != longest_frag:
                del possibles[i]
                del possibles_info[i]

        n = len(possibles)

        if n == 0:
            # None found
            return None

        if n == 1:
            # Exactly one found
            return (possibles[0], possibles_info[0])

        # Multiple were found. Try and pick one.

        if n == 2 and possibles[0].frags[0] == possibles[1].frags[0]:
            # Special, common case where we can override the addressing mode
            # The mode might be forced with "<" or ">"
            # If not, we might have the address defined. If so, pick based on value.

            if '>' in text:
                # Developer says two-bytes
                sz = 2
            elif '<' in text:
                # Developer says one-byte
                sz = 1
            else:
                # We might be able to figure out what it is if the value is
                # defined already
                targ = None
                for p in possibles_info:
                    for k in p:
                        if targ is None:
                            targ = p[k]
                            continue
                        if p[k] != targ:
                            raise AssemblyException('Could not from possible opcodes')

                try:
                    tv = assembler.parse_numeric(targ)
                    if tv < 256:
                        sz = 1
                    else:
                        sz = 2
                except Exception:
                    # Can't tell if this is one or two bytes ... go with 2
                    sz = 2

            for i in range(len(possibles) - 1, -1, -1):
                cnt = 0
                for f in possibles[i].code:
                    if isinstance(f, str):
                        cnt += 1
                if cnt == sz:
                    # The Z80 includes repeats like 2A and ED6B. TODO: fix this
                    return(possibles[i], possibles_info[i])

            return (possibles[0], possibles_info[0])

        # Their might be multiple opcodes with the same mnemonics. In that case,
        # just pick the first

        g = possibles[0]
        for p in possibles:
            if p.mnemonic != g.mnemonic:
                raise AssemblyException('Multiple Matches')
        return(possibles[i], possibles_info[i])

    def fill_in_opcode(self, _text, asm, address, op, pass_number):
        '''
        '''

        opcode = op[0]
        info = op[1]
        if pass_number == 0:
            return [0] * len(opcode.code)
        else:
            for key in info:
                val = asm.parse_numeric(str(info[key]))
                info[key] = [info[key], val]
            ret = []
            for c in opcode.code:
                if isinstance(c, str):
                    numval = info[c[0]][1]
                    if 's2' in opcode.use[c[0]]:
                        sz = 2
                    else:
                        sz = 1
                    if 'pcr' in opcode.use[key]:
                        numval = numval - address - len(opcode.code)
                        if sz == 2:
                            if numval < -32768 or numval > 32767:
                                raise AssemblyException('Destination is too far away')
                            if numval < 0:
                                numval = numval + 65536
                        else:
                            if numval < -128 or numval > 127:
                                raise AssemblyException('Destination is too far away')
                            if numval < 0:
                                numval = numval + 256

                    if sz == 1 and numval > 255:
                        raise AssemblyException('Value is larger than one byte')

                    if c[1] == '1':
                        ret.append((numval >> 8) & 0xFF)
                    else:
                        ret.append(numval & 0xFF)

                else:
                    ret.append(c)

            return ret



from typing import List


def load_binary(names: str) -> List[int]:
    '''Read a group of binary files

    For example "a.bin + b.bin+c.bin"

    Args:
        names (str) : the group of names
    Returns:
        List[int] : the binary data
    '''

    names = names.split('+')
    ret = []
    for n in names:
        n = n.strip()
        with open(n, 'rb') as f:
            data = f.read()
            for d in data:
                ret.append(int(d))

    return ret



import opcodetools.cpu.base_cpu

OPCODES = [
    {"mnemonic": "NOP", "code": "01", "use": ""             },
    {"mnemonic": "LSRD", "code": "04", "use": ""             },
    {"mnemonic": "TAP", "code": "06", "use": ""             },
    {"mnemonic": "TPA", "code": "07", "use": ""             },
    {"mnemonic": "INX", "code": "08", "use": ""             },
    {"mnemonic": "DEX", "code": "09", "use": ""             },
    {"mnemonic": "CLV", "code": "0A", "use": ""             },
    {"mnemonic": "SEV", "code": "0B", "use": ""             },
    {"mnemonic": "CLC", "code": "0C", "use": ""             },
    {"mnemonic": "SEC", "code": "0D", "use": ""             },
    {"mnemonic": "CLI", "code": "0E", "use": ""             },
    {"mnemonic": "SEI", "code": "0F", "use": ""             },
    {"mnemonic": "SBA", "code": "10", "use": ""             },
    {"mnemonic": "CBA", "code": "11", "use": ""             },
    {"mnemonic": "TAB", "code": "16", "use": ""             },
    {"mnemonic": "TBA", "code": "17", "use": ""             },
    {"mnemonic": "DAA", "code": "19", "use": ""             },
    {"mnemonic": "ABA", "code": "1B", "use": ""             },
    {"mnemonic": "BRA r", "code": "20rr", "use": "r=code_pcr"   },
    {"mnemonic": "BRN r", "code": "21rr", "use": "r=code_pcr"   },
    {"mnemonic": "BHI r", "code": "22rr", "use": "r=code_pcr"   },
    {"mnemonic": "BLS r", "code": "23rr", "use": "r=code_pcr"   },
    {"mnemonic": "BCC r", "code": "24rr", "use": "r=code_pcr"   },
    {"mnemonic": "BHS r", "code": "24rr", "use": "r=code_pcr"   },
    {"mnemonic": "BLO r", "code": "25rr", "use": "r=code_pcr"   },
    {"mnemonic": "BCS r", "code": "25rr", "use": "r=code_pcr"   },
    {"mnemonic": "BNE r", "code": "26rr", "use": "r=code_pcr"   },
    {"mnemonic": "BEQ r", "code": "27rr", "use": "r=code_pcr"   },
    {"mnemonic": "BVC r", "code": "28rr", "use": "r=code_pcr"   },
    {"mnemonic": "BVS r", "code": "29rr", "use": "r=code_pcr"   },
    {"mnemonic": "BPL r", "code": "2Arr", "use": "r=code_pcr"   },
    {"mnemonic": "BMI r", "code": "2Brr", "use": "r=code_pcr"   },
    {"mnemonic": "BGE r", "code": "2Crr", "use": "r=code_pcr"   },
    {"mnemonic": "BLT r", "code": "2Drr", "use": "r=code_pcr"   },
    {"mnemonic": "BGT r", "code": "2Err", "use": "r=code_pcr"   },
    {"mnemonic": "BLE r", "code": "2Frr", "use": "r=code_pcr"   },
    {"mnemonic": "TSX", "code": "30", "use": ""             },
    {"mnemonic": "INS", "code": "31", "use": ""             },
    {"mnemonic": "PULA", "code": "32", "use": ""             },
    {"mnemonic": "PULB", "code": "33", "use": ""             },
    {"mnemonic": "DES", "code": "34", "use": ""             },
    {"mnemonic": "TXS", "code": "35", "use": ""             },
    {"mnemonic": "PSHA", "code": "36", "use": ""             },
    {"mnemonic": "PSHB", "code": "37", "use": ""             },
    {"mnemonic": "PULX", "code": "38", "use": ""             },
    {"mnemonic": "RTS", "code": "39", "use": ""             },
    {"mnemonic": "ABX", "code": "3A", "use": ""             },
    {"mnemonic": "RTI", "code": "3B", "use": ""             },
    {"mnemonic": "PSHX", "code": "3C", "use": ""             },
    {"mnemonic": "MUL", "code": "3D", "use": ""             },
    {"mnemonic": "WAI", "code": "3E", "use": ""             },
    {"mnemonic": "SWI", "code": "3F", "use": ""             },
    {"mnemonic": "NEGA", "code": "40", "use": ""             },
    {"mnemonic": "COMA", "code": "43", "use": ""             },
    {"mnemonic": "LSRA", "code": "44", "use": ""             },
    {"mnemonic": "RORA", "code": "46", "use": ""             },
    {"mnemonic": "ASRA", "code": "47", "use": ""             },
    {"mnemonic": "LSLA", "code": "48", "use": ""             },
    {"mnemonic": "ASLA", "code": "48", "use": ""             },
    {"mnemonic": "ROLA", "code": "49", "use": ""             },
    {"mnemonic": "DECA", "code": "4A", "use": ""             },
    {"mnemonic": "INCA", "code": "4C", "use": ""             },
    {"mnemonic": "TSTA", "code": "4D", "use": ""             },
    {"mnemonic": "CLRA", "code": "4F", "use": ""             },
    {"mnemonic": "NEGB", "code": "50", "use": ""             },
    {"mnemonic": "COMB", "code": "53", "use": ""             },
    {"mnemonic": "LSRB", "code": "54", "use": ""             },
    {"mnemonic": "RORB", "code": "56", "use": ""             },
    {"mnemonic": "ASRB", "code": "57", "use": ""             },
    {"mnemonic": "LSLB", "code": "58", "use": ""             },
    {"mnemonic": "ASLB", "code": "58", "use": ""             },
    {"mnemonic": "ROLB", "code": "59", "use": ""             },
    {"mnemonic": "DECB", "code": "5A", "use": ""             },
    {"mnemonic": "INCB", "code": "5C", "use": ""             },
    {"mnemonic": "TSTB", "code": "5D", "use": ""             },
    {"mnemonic": "CLRB", "code": "5F", "use": ""             },
    {"mnemonic": "NEG i,X", "code": "60ii", "use": "i=const"      },
    {"mnemonic": "COM i,X", "code": "63ii", "use": "i=const"      },
    {"mnemonic": "LSR i,X", "code": "64ii", "use": "i=const"      },
    {"mnemonic": "ROR i,X", "code": "66ii", "use": "i=const"      },
    {"mnemonic": "ASR i,X", "code": "67ii", "use": "i=const"      },
    {"mnemonic": "ASL i,X", "code": "68ii", "use": "i=const"      },
    {"mnemonic": "LSL i,X", "code": "68ii", "use": "i=const"      },
    {"mnemonic": "ROL i,X", "code": "69ii", "use": "i=const"      },
    {"mnemonic": "DEC i,X", "code": "6Aii", "use": "i=const"      },
    {"mnemonic": "INC i,X", "code": "6Cii", "use": "i=const"      },
    {"mnemonic": "TST i,X", "code": "6Dii", "use": "i=const"      },
    {"mnemonic": "JMP i,X", "code": "6Eii", "use": "i=const"      },
    {"mnemonic": "CLR i,X", "code": "6Fii", "use": "i=const"      },
    {"mnemonic": "NEG t", "code": "70t1t0", "use": "t=data_rw"    },
    {"mnemonic": "COM t", "code": "73t1t0", "use": "t=data_rw"    },
    {"mnemonic": "LSR t", "code": "74t1t0", "use": "t=data_rw"    },
    {"mnemonic": "ROR t", "code": "76t1t0", "use": "t=data_rw"    },
    {"mnemonic": "ASR t", "code": "77t1t0", "use": "t=data_rw"    },
    {"mnemonic": "ASL t", "code": "78t1t0", "use": "t=data_rw"    },
    {"mnemonic": "LSL t", "code": "78t1t0", "use": "t=data_rw"    },
    {"mnemonic": "ROL t", "code": "79t1t0", "use": "t=data_rw"    },
    {"mnemonic": "DEC t", "code": "7At1t0", "use": "t=data_rw"    },
    {"mnemonic": "INC t", "code": "7Ct1t0", "use": "t=data_rw"    },
    {"mnemonic": "TST t", "code": "7Dt1t0", "use": "t=data_r"     },
    {"mnemonic": "JMP t", "code": "7Et1t0", "use": "t=code"       },
    {"mnemonic": "CLR t", "code": "7Ft1t0", "use": "t=data_w"     },
    {"mnemonic": "SUBA #b", "code": "80bb", "use": "b=const"      },
    {"mnemonic": "CMPA #b", "code": "81bb", "use": "b=const"      },
    {"mnemonic": "SBCA #b", "code": "82bb", "use": "b=const"      },
    {"mnemonic": "SUBD #w", "code": "83w1w0", "use": "w=const"      },
    {"mnemonic": "ANDA #b", "code": "84bb", "use": "b=const"      },
    {"mnemonic": "BITA #b", "code": "85bb", "use": "b=const"      },
    {"mnemonic": "LDA #b", "code": "86bb", "use": "b=const"      },
    {"mnemonic": "EORA #b", "code": "88bb", "use": "b=const"      },
    {"mnemonic": "ADCA #b", "code": "89bb", "use": "b=const"      },
    {"mnemonic": "ORA #b", "code": "8Abb", "use": "b=const"      },
    {"mnemonic": "ADDA #b", "code": "8Bbb", "use": "b=const"      },
    {"mnemonic": "CPX #w", "code": "8Cw1w0", "use": "w=const"      },
    {"mnemonic": "BSR r", "code": "8Drr", "use": "r=code_pcr"   },
    {"mnemonic": "LDS #w", "code": "8Ew1w0", "use": "w=const"      },
    {"mnemonic": "SUBA p", "code": "90pp", "use": "p=data_r"     },
    {"mnemonic": "CMPA p", "code": "91pp", "use": "p=data_r"     },
    {"mnemonic": "SBCA p", "code": "92pp", "use": "p=data_r"     },
    {"mnemonic": "SUBD p", "code": "93pp", "use": "p=data_r"     },
    {"mnemonic": "ANDA p", "code": "94pp", "use": "p=data_r"     },
    {"mnemonic": "BITA p", "code": "95pp", "use": "p=data_r"     },
    {"mnemonic": "LDA p", "code": "96pp", "use": "p=data_r"     },
    {"mnemonic": "STA p", "code": "97pp", "use": "p=data_w"     },
    {"mnemonic": "EORA p", "code": "98pp", "use": "p=data_r"     },
    {"mnemonic": "ADCA p", "code": "99pp", "use": "p=data_r"     },
    {"mnemonic": "ORA p", "code": "9App", "use": "p=data_r"     },
    {"mnemonic": "ADDA p", "code": "9Bpp", "use": "p=data_r"     },
    {"mnemonic": "CPX p", "code": "9Cpp", "use": "p=data_r"     },
    {"mnemonic": "JSR p", "code": "9Dpp", "use": "p=code"       },
    {"mnemonic": "LDS p", "code": "9Epp", "use": "p=data_r"     },
    {"mnemonic": "STS p", "code": "9Fpp", "use": "p=data_w"     },
    {"mnemonic": "SUBA i,X", "code": "A0ii", "use": "i=const"      },
    {"mnemonic": "CMPA i,X", "code": "A1ii", "use": "i=const"      },
    {"mnemonic": "SBCA i,X", "code": "A2ii", "use": "i=const"      },
    {"mnemonic": "SUBD i,X", "code": "A3ii", "use": "i=const"      },
    {"mnemonic": "ANDA i,X", "code": "A4ii", "use": "i=const"      },
    {"mnemonic": "BITA i,X", "code": "A5ii", "use": "i=const"      },
    {"mnemonic": "LDA i,X", "code": "A6ii", "use": "i=const"      },
    {"mnemonic": "STA i,X", "code": "A7ii", "use": "i=const"      },
    {"mnemonic": "EORA i,X", "code": "A8ii", "use": "i=const"      },
    {"mnemonic": "ADCA i,X", "code": "A9ii", "use": "i=const"      },
    {"mnemonic": "ORA i,X", "code": "AAii", "use": "i=const"      },
    {"mnemonic": "ADDA i,X", "code": "ABii", "use": "i=const"      },
    {"mnemonic": "CPX i,X", "code": "ACii", "use": "i=const"      },
    {"mnemonic": "JSR i,X", "code": "ADii", "use": "i=const"      },
    {"mnemonic": "LDS i,X", "code": "AEii", "use": "i=const"      },
    {"mnemonic": "STS i,X", "code": "AFii", "use": "i=const"      },
    {"mnemonic": "SUBA t", "code": "B0t1t0", "use": "t=data_r"     },
    {"mnemonic": "CMPA t", "code": "B1t1t0", "use": "t=data_r"     },
    {"mnemonic": "SBCA t", "code": "B2t1t0", "use": "t=data_r"     },
    {"mnemonic": "SUBD t", "code": "B3t1t0", "use": "t=data_r"     },
    {"mnemonic": "ANDA t", "code": "B4t1t0", "use": "t=data_r"     },
    {"mnemonic": "BITA t", "code": "B5t1t0", "use": "t=data_r"     },
    {"mnemonic": "LDA t", "code": "B6t1t0", "use": "t=data_r"     },
    {"mnemonic": "STA t", "code": "B7t1t0", "use": "t=data_w"     },
    {"mnemonic": "EORA t", "code": "B8t1t0", "use": "t=data_r"     },
    {"mnemonic": "ADCA t", "code": "B9t1t0", "use": "t=data_r"     },
    {"mnemonic": "ORA t", "code": "BAt1t0", "use": "t=data_r"     },
    {"mnemonic": "ADDA t", "code": "BBt1t0", "use": "t=data_r"     },
    {"mnemonic": "CPX t", "code": "BCt1t0", "use": "t=data_r"     },
    {"mnemonic": "JSR t", "code": "BDt1t0", "use": "t=code"       },
    {"mnemonic": "LDS t", "code": "BEt1t0", "use": "t=data_r"     },
    {"mnemonic": "STS t", "code": "BFt1t0", "use": "t=data_w"     },
    {"mnemonic": "SUBB #b", "code": "C0bb", "use": "b=const"      },
    {"mnemonic": "CMPB #b", "code": "C1bb", "use": "b=const"      },
    {"mnemonic": "SBCB #b", "code": "C2bb", "use": "b=const"      },
    {"mnemonic": "ADDD #w", "code": "C3w1w0", "use": "w=const"      },
    {"mnemonic": "ANDB #b", "code": "C4bb", "use": "b=const"      },
    {"mnemonic": "BITB #b", "code": "C5bb", "use": "b=const"      },
    {"mnemonic": "LDB #b", "code": "C6bb", "use": "b=const"      },
    {"mnemonic": "EORB #b", "code": "C8bb", "use": "b=const"      },
    {"mnemonic": "ADCB #b", "code": "C9bb", "use": "b=const"      },
    {"mnemonic": "ORB #b", "code": "CAbb", "use": "b=const"      },
    {"mnemonic": "ADDB #b", "code": "CBbb", "use": "b=const"      },
    {"mnemonic": "LDD #w", "code": "CCw1w0", "use": "w=const"      },
    {"mnemonic": "LDX #w", "code": "CEw1w0", "use": "w=const"      },
    {"mnemonic": "SUBB p", "code": "D0pp", "use": "p=data_r"     },
    {"mnemonic": "CMPB p", "code": "D1pp", "use": "p=data_r"     },
    {"mnemonic": "SBCB p", "code": "D2pp", "use": "p=data_r"     },
    {"mnemonic": "ADDD p", "code": "D3pp", "use": "p=data_r"     },
    {"mnemonic": "ANDB p", "code": "D4pp", "use": "p=data_r"     },
    {"mnemonic": "BITB p", "code": "D5pp", "use": "p=data_r"     },
    {"mnemonic": "LDB p", "code": "D6pp", "use": "p=data_r"     },
    {"mnemonic": "STB p", "code": "D7pp", "use": "p=data_w"     },
    {"mnemonic": "EORB p", "code": "D8pp", "use": "p=data_r"     },
    {"mnemonic": "ADCB p", "code": "D9pp", "use": "p=data_r"     },
    {"mnemonic": "ORB p", "code": "DApp", "use": "p=data_r"     },
    {"mnemonic": "ADDB p", "code": "DBpp", "use": "p=data_r"     },
    {"mnemonic": "LDD p", "code": "DCpp", "use": "p=data_r"     },
    {"mnemonic": "STD p", "code": "DDpp", "use": "p=data_w"     },
    {"mnemonic": "LDX p", "code": "DEpp", "use": "p=data_r"     },
    {"mnemonic": "STX p", "code": "DFpp", "use": "p=data_r"     },
    {"mnemonic": "SUBB i,X", "code": "E0ii", "use": "i=const"      },
    {"mnemonic": "CMPB i,X", "code": "E1ii", "use": "i=const"      },
    {"mnemonic": "SBCB i,X", "code": "E2ii", "use": "i=const"      },
    {"mnemonic": "ADDD i,X", "code": "E3ii", "use": "i=const"      },
    {"mnemonic": "ANDB i,X", "code": "E4ii", "use": "i=const"      },
    {"mnemonic": "BITB i,X", "code": "E5ii", "use": "i=const"      },
    {"mnemonic": "LDB i,X", "code": "E6ii", "use": "i=const"      },
    {"mnemonic": "STB i,X", "code": "E7ii", "use": "i=const"      },
    {"mnemonic": "EORB i,X", "code": "E8ii", "use": "i=const"      },
    {"mnemonic": "ADCB i,X", "code": "E9ii", "use": "i=const"      },
    {"mnemonic": "ORB i,X", "code": "EAii", "use": "i=const"      },
    {"mnemonic": "ADDB i,X", "code": "EBii", "use": "i=const"      },
    {"mnemonic": "LDD i,X", "code": "ECii", "use": "i=const"      },
    {"mnemonic": "STD i,X", "code": "EDii", "use": "i=const"      },
    {"mnemonic": "LDX i,X", "code": "EEii", "use": "i=const"      },
    {"mnemonic": "STX i,X", "code": "EFii", "use": "i=const"      },
    {"mnemonic": "SUBB t", "code": "F0t1t0", "use": "t=data_r"     },
    {"mnemonic": "CMPB t", "code": "F1t1t0", "use": "t=data_r"     },
    {"mnemonic": "SBCB t", "code": "F2t1t0", "use": "t=data_r"     },
    {"mnemonic": "ADDD t", "code": "F3t1t0", "use": "t=data_r"     },
    {"mnemonic": "ANDB t", "code": "F4t1t0", "use": "t=data_r"     },
    {"mnemonic": "BITB t", "code": "F5t1t0", "use": "t=data_r"     },
    {"mnemonic": "LDB t", "code": "F6t1t0", "use": "t=data_r"     },
    {"mnemonic": "STB t", "code": "F7t1t0", "use": "t=data_w"     },
    {"mnemonic": "EORB t", "code": "F8t1t0", "use": "t=data_r"     },
    {"mnemonic": "ADCB t", "code": "F9t1t0", "use": "t=data_r"     },
    {"mnemonic": "ORB t", "code": "FAt1t0", "use": "t=data_r"     },
    {"mnemonic": "ADDB t", "code": "FBt1t0", "use": "t=data_r"     },
    {"mnemonic": "LDD t", "code": "FCt1t0", "use": "t=data_r"     },
    {"mnemonic": "STD t", "code": "FDt1t0", "use": "t=data_w"     },
    {"mnemonic": "LDX t", "code": "FEt1t0", "use": "t=data_r"     },
    {"mnemonic": "STX t", "code": "FFt1t0", "use": "t=data_w"     },
]


class CPU_6803(opcodetools.cpu.base_cpu.CPU):

    def __init__(self):
        super().__init__(OPCODES)



import opcodetools.cpu.base_cpu

OPCODES = [
    {"mnemonic": "VEC SCALE=0, BRI=b, X=x, Y=y",
        "code": "0000_0yyy_yyyy_yyyy bbbb_0xxx_xxxx_xxxx", "use": ""},
    {"mnemonic": "VEC SCALE=1, BRI=b, X=x, Y=y",
        "code": "0001_0yyy_yyyy_yyyy bbbb_0xxx_xxxx_xxxx", "use": ""},
    {"mnemonic": "VEC SCALE=2, BRI=b, X=x, Y=y",
        "code": "0010_0yyy_yyyy_yyyy bbbb_0xxx_xxxx_xxxx", "use": ""},
    {"mnemonic": "VEC SCALE=3, BRI=b, X=x, Y=y",
        "code": "0011_0yyy_yyyy_yyyy bbbb_0xxx_xxxx_xxxx", "use": ""},
    {"mnemonic": "VEC SCALE=4, BRI=b, X=x, Y=y",
        "code": "0100_0yyy_yyyy_yyyy bbbb_0xxx_xxxx_xxxx", "use": ""},
    {"mnemonic": "VEC SCALE=5, BRI=b, X=x, Y=y",
        "code": "0101_0yyy_yyyy_yyyy bbbb_0xxx_xxxx_xxxx", "use": ""},
    {"mnemonic": "VEC SCALE=6, BRI=b, X=x, Y=y",
        "code": "0110_0yyy_yyyy_yyyy bbbb_0xxx_xxxx_xxxx", "use": ""},
    {"mnemonic": "VEC SCALE=7, BRI=b, X=x, Y=y",
        "code": "0111_0yyy_yyyy_yyyy bbbb_0xxx_xxxx_xxxx", "use": ""},
    {"mnemonic": "VEC SCALE=8, BRI=b, X=x, Y=y",
        "code": "1000_0yyy_yyyy_yyyy bbbb_0xxx_xxxx_xxxx", "use": ""},
    {"mnemonic": "VEC SCALE=9, BRI=b, X=x, Y=y",
        "code": "1001_0yyy_yyyy_yyyy bbbb_0xxx_xxxx_xxxx", "use": ""},

    {"mnemonic": "CUR SCALE=s, X=t, Y=u",
        "code": "1010_00yy_yyyy_yyyy ssss_00xx_xxxx_xxxx", "use": ""},

    {"mnemonic": "HALT", "code": "1011_0000_0000_0000", "use": ""},

    {"mnemonic": "JSR a", "code": "1100_aaaa_aaaa_aaaa", "use": "a=code"},

    {"mnemonic": "RTS", "code": "1101_0000_bbbb_bbbb", "use": ""},

    {"mnemonic": "JMP a", "code": "1110_aaaa_aaaa_aaaa", "use": "a=code"},

    {"mnemonic": "SVEC SCALE=c, BRI=b, X=d, Y=y", "code": "1111_gyyy_bbbb_hxxx",
        "details": "h is upper bit of c, g is lower bit", "use": ""}
]

'''
      aa bb
0858: DB F0

      Xxx       Yyy
1101_1011 1111_0000

y = 0 *256 = 0
x = 3 *256 = 768
'''


class CPU_DVG(opcodetools.cpu.base_cpu.CPU):

    def __init__(self):        
        self._dvg_copcodes = []

        for entry in OPCODES:
            entry['code'] = entry['code'].replace('_', '').replace(' ', '')
            # Note that these are BIT fields ... not BYTE fields like other
            # CPUs
            self._dvg_copcodes.append(entry)
            
        super().__init__(self._dvg_copcodes)
            
    def find_opcodes_for_binary(self, binary: list, exact: bool=False) -> list:
        
        if not exact:
            raise Exception('Still working on a generic DVG disassembler')
        
        leading = binary[1] >> 4
        return [ self._opcodes[leading] ]
    



import sys

from opcodetools.disassembler import binary
from opcodetools.cpu import cpu_manager

# py dasm Z80 1000 "a.bin+b.bin+c.bin"

cpuname = sys.argv[1]
org = int(sys.argv[2], 16)
names = sys.argv[3]

bindata = binary.load_binary(names)

cpu = cpu_manager.get_cpu_by_name(cpuname)

print('; CPU:', cpuname)
print('; ORIGIN:', hex(org))
print('; FILES:', names)
print()

pos = 0

while pos < len(bindata):
    ops = cpu.find_opcodes_for_binary(bindata[pos:])
    if len(ops) != 1:
        print(cpu.binary_to_string_unknown(pos + org, bindata[pos]))
        pos += 1
        continue

    opc = ops[0]

    dat = bindata[pos:pos + len(opc.code)]
    fills = cpu.get_mnemonic_fills(ops[0], dat, pos + org)
    s = cpu.binary_to_string(opc, dat, pos + org, fills)
    pos += len(dat)

    print(s)



import opcodetools.cpu.base_cpu


# http://www.keil.com/support/man/docs/is51/is51_opcodes.htm
OPCODES = [
    {"mnemonic": "NOP",              "code": "00",               "use": ""                         },
    {"mnemonic": "AJMP p",           "code": "01pp",             "use": "p=code_11"                },
    {"mnemonic": "LJMP t",           "code": "02t1t0",           "use": "t=code"                   },
    {"mnemonic": "RR A",             "code": "03",               "use": ""                         },
    {"mnemonic": "INC A",            "code": "04",               "use": ""                         },
    {"mnemonic": "INC p",            "code": "05pp",             "use": "p=data_rw"                },
    {"mnemonic": "INC @R0",          "code": "06",               "use": ""                         },
    {"mnemonic": "INC @R1",          "code": "07",               "use": ""                         },
    {"mnemonic": "INC R0",           "code": "08",               "use": ""                         },
    {"mnemonic": "INC R1",           "code": "09",               "use": ""                         },
    {"mnemonic": "INC R2",           "code": "0A",               "use": ""                         },
    {"mnemonic": "INC R3",           "code": "0B",               "use": ""                         },
    {"mnemonic": "INC R4",           "code": "0C",               "use": ""                         },
    {"mnemonic": "INC R5",           "code": "0D",               "use": ""                         },
    {"mnemonic": "INC R6",           "code": "0E",               "use": ""                         },
    {"mnemonic": "INC R7",           "code": "0F",               "use": ""                         },

    {"mnemonic": "JBC b,r",          "code": "10bbrr",           "use": "b=data_bit_rw,r=code_pcr" },
    {"mnemonic": "ACALL p",          "code": "11pp",             "use": "p=code_11"                },
    {"mnemonic": "LCALL t",          "code": "12t1t0",           "use": "t=code"                   },
    {"mnemonic": "RRC A",            "code": "13",               "use": ""                         },
    {"mnemonic": "DEC A",            "code": "14",               "use": ""                         },
    {"mnemonic": "DEC p",            "code": "15pp",             "use": "p=data_rw"                },
    {"mnemonic": "DEC @R0",          "code": "16",               "use": ""                         },
    {"mnemonic": "DEC @R1",          "code": "17",               "use": ""                         },
    {"mnemonic": "DEC R0",           "code": "18",               "use": ""                         },
    {"mnemonic": "DEC R1",           "code": "19",               "use": ""                         },
    {"mnemonic": "DEC R2",           "code": "1A",               "use": ""                         },
    {"mnemonic": "DEC R3",           "code": "1B",               "use": ""                         },
    {"mnemonic": "DEC R4",           "code": "1C",               "use": ""                         },
    {"mnemonic": "DEC R5",           "code": "1D",               "use": ""                         },
    {"mnemonic": "DEC R6",           "code": "1E",               "use": ""                         },
    {"mnemonic": "DEC R7",           "code": "1F",               "use": ""                         },

    {"mnemonic": "JB b,r",           "code": "20bbrr",           "use": "b=data_bit_r,r=code_pcr"  },
    {"mnemonic": "AJMP p",           "code": "21pp",             "use": "p=code_11"                },
    {"mnemonic": "RET",              "code": "22",               "use": ""                         },
    {"mnemonic": "RL A",             "code": "23",               "use": ""                         },
    {"mnemonic": "ADD A,#b",         "code": "24bb",             "use": "b=const"                  },
    {"mnemonic": "ADD A,p",          "code": "25pp",             "use": "p=data_r"                 },
    {"mnemonic": "ADD A,@R0",        "code": "26",               "use": ""                         },
    {"mnemonic": "ADD A,@R1",        "code": "27",               "use": ""                         },
    {"mnemonic": "ADD A,R0",         "code": "28",               "use": ""                         },
    {"mnemonic": "ADD A,R1",         "code": "29",               "use": ""                         },
    {"mnemonic": "ADD A,R2",         "code": "2A",               "use": ""                         },
    {"mnemonic": "ADD A,R3",         "code": "2B",               "use": ""                         },
    {"mnemonic": "ADD A,R4",         "code": "2C",               "use": ""                         },
    {"mnemonic": "ADD A,R5",         "code": "2D",               "use": ""                         },
    {"mnemonic": "ADD A,R6",         "code": "2E",               "use": ""                         },
    {"mnemonic": "ADD A,R7",         "code": "2F",               "use": ""                         },

    {"mnemonic": "JNB b,r",          "code": "30bbrr",           "use": "b=data_bit_r,r=code_pcr"  },
    {"mnemonic": "ACALL p",          "code": "31pp",             "use": "p=code_11"                },
    {"mnemonic": "RETI",             "code": "32",               "use": ""                         },
    {"mnemonic": "RLC A",            "code": "33",               "use": ""                         },
    {"mnemonic": "ADDC A,#b",        "code": "34bb",             "use": "b=const"                  },
    {"mnemonic": "ADDC A,p",         "code": "35pp",             "use": "p=data_r"                 },
    {"mnemonic": "ADDC A,@R0",       "code": "36",               "use": ""                         },
    {"mnemonic": "ADDC A,@R1",       "code": "37",               "use": ""                         },
    {"mnemonic": "ADDC A,R0",        "code": "38",               "use": ""                         },
    {"mnemonic": "ADDC A,R1",        "code": "39",               "use": ""                         },
    {"mnemonic": "ADDC A,R2",        "code": "3A",               "use": ""                         },
    {"mnemonic": "ADDC A,R3",        "code": "3B",               "use": ""                         },
    {"mnemonic": "ADDC A,R4",        "code": "3C",               "use": ""                         },
    {"mnemonic": "ADDC A,R5",        "code": "3D",               "use": ""                         },
    {"mnemonic": "ADDC A,R6",        "code": "3E",               "use": ""                         },
    {"mnemonic": "ADDC A,R7",        "code": "3F",               "use": ""                         },

    {"mnemonic": "JC r",             "code": "40rr",             "use": "r=code_pcr"               },
    {"mnemonic": "AJMP p",           "code": "41pp",             "use": "p=code_11"                },
    {"mnemonic": "ORL p,A",          "code": "42pp",             "use": "p=data_rw"                },
    {"mnemonic": "ORL p,#b",         "code": "43ppbb",           "use": "p=data_rw,b=const"        },
    {"mnemonic": "ORL A,#b",         "code": "44bb",             "use": "b=const"                  },
    {"mnemonic": "ORL A,p",          "code": "45pp",             "use": "p=data_r"                 },
    {"mnemonic": "ORL A,@R0",        "code": "46",               "use": ""                         },
    {"mnemonic": "ORL A,@R1",        "code": "47",               "use": ""                         },
    {"mnemonic": "ORL A,R0",         "code": "48",               "use": ""                         },
    {"mnemonic": "ORL A,R1",         "code": "49",               "use": ""                         },
    {"mnemonic": "ORL A,R2",         "code": "4A",               "use": ""                         },
    {"mnemonic": "ORL A,R3",         "code": "4B",               "use": ""                         },
    {"mnemonic": "ORL A,R4",         "code": "4C",               "use": ""                         },
    {"mnemonic": "ORL A,R5",         "code": "4D",               "use": ""                         },
    {"mnemonic": "ORL A,R6",         "code": "4E",               "use": ""                         },
    {"mnemonic": "ORL A,R7",         "code": "4F",               "use": ""                         },

    {"mnemonic": "JNC r",            "code": "50rr",             "use": "r=code_pcr"               },
    {"mnemonic": "ACALL p",          "code": "51pp",             "use": "p=code_11"                },
    {"mnemonic": "ANL p,A",          "code": "52pp",             "use": "p=data_rw"                },
    {"mnemonic": "ANL p,#b",         "code": "53ppbb",           "use": "p=data_rw,b=const"        },
    {"mnemonic": "ANL A,#b",         "code": "54bb",             "use": "b=const"                  },
    {"mnemonic": "ANL A,p",          "code": "55pp",             "use": "p=data_r"                 },
    {"mnemonic": "ANL @R0",          "code": "56",               "use": ""                         },
    {"mnemonic": "ANL @R1",          "code": "57",               "use": ""                         },
    {"mnemonic": "ANL A,R0",         "code": "58",               "use": ""                         },
    {"mnemonic": "ANL A,R1",         "code": "59",               "use": ""                         },
    {"mnemonic": "ANL A,R2",         "code": "5A",               "use": ""                         },
    {"mnemonic": "ANL A,R3",         "code": "5B",               "use": ""                         },
    {"mnemonic": "ANL A,R4",         "code": "5C",               "use": ""                         },
    {"mnemonic": "ANL A,R5",         "code": "5D",               "use": ""                         },
    {"mnemonic": "ANL A,R6",         "code": "5E",               "use": ""                         },
    {"mnemonic": "ANL A,R7",         "code": "5F",               "use": ""                         },

    {"mnemonic": "JZ r",             "code": "60rr",             "use": "r=code_pcr"               },
    {"mnemonic": "AJMP p",           "code": "61pp",             "use": "p=code_11"                },
    {"mnemonic": "XRL p,A",          "code": "62pp",             "use": "p=data_rw"                },
    {"mnemonic": "XRL p,#b",         "code": "63ppbb",           "use": "p=data_rw,b=const"        },
    {"mnemonic": "XRL A,#b",         "code": "64bb",             "use": "b=const"                  },
    {"mnemonic": "XRL A,p",          "code": "65pp",             "use": "p=data_r"                 },
    {"mnemonic": "XRL A,@R0",        "code": "66",               "use": ""                         },
    {"mnemonic": "XRL A,@R1",        "code": "67",               "use": ""                         },
    {"mnemonic": "XRL A,R0",         "code": "68",               "use": ""                         },
    {"mnemonic": "XRL A,R1",         "code": "69",               "use": ""                         },
    {"mnemonic": "XRL A,R2",         "code": "6A",               "use": ""                         },
    {"mnemonic": "XRL A,R3",         "code": "6B",               "use": ""                         },
    {"mnemonic": "XRL A,R4",         "code": "6C",               "use": ""                         },
    {"mnemonic": "XRL A,R5",         "code": "6D",               "use": ""                         },
    {"mnemonic": "XRL A,R6",         "code": "6E",               "use": ""                         },
    {"mnemonic": "XRL A,R7",         "code": "6F",               "use": ""                         },

    {"mnemonic": "JNZ r",            "code": "70rr",             "use": "r=code_pcr"               },
    {"mnemonic": "ACALL p",          "code": "71pp",             "use": "p=code_11"                },
    {"mnemonic": "ORL C,b",          "code": "72bb",             "use": "b=data_bit_r"             },
    {"mnemonic": "JMP @A+DPTR",      "code": "73",               "use": ""                         },
    {"mnemonic": "MOV A,#b",         "code": "74bb",             "use": "b=const"                  },
    {"mnemonic": "MOV p,#b",         "code": "75ppbb",           "use": "p=data_w,b=const"         },
    {"mnemonic": "MOV @R0,#b",       "code": "76bb",             "use": "b=const"                  },
    {"mnemonic": "MOV @R1,#b",       "code": "77bb",             "use": "b=const"                  },
    {"mnemonic": "MOV R0,#b",        "code": "78bb",             "use": "b=const"                  },
    {"mnemonic": "MOV R1,#b",        "code": "79bb",             "use": "b=const"                  },
    {"mnemonic": "MOV R2,#b",        "code": "7Abb",             "use": "b=const"                  },
    {"mnemonic": "MOV R3,#b",        "code": "7Bbb",             "use": "b=const"                  },
    {"mnemonic": "MOV R4,#b",        "code": "7Cbb",             "use": "b=const"                  },
    {"mnemonic": "MOV R5,#b",        "code": "7Dbb",             "use": "b=const"                  },
    {"mnemonic": "MOV R6,#b",        "code": "7Ebb",             "use": "b=const"                  },
    {"mnemonic": "MOV R7,#b",        "code": "7Fbb",             "use": "b=const"                  },

    {"mnemonic": "SJMP r",           "code": "80rr",             "use": "r=code_pcr"               },
    {"mnemonic": "AJMP p",           "code": "81pp",             "use": "p=code_11"                },
    {"mnemonic": "ANL C,b",          "code": "82bb",             "use": "b=data_bit_r"             },
    {"mnemonic": "MOVC A,@A+PC",     "code": "83",               "use": ""                         },
    {"mnemonic": "DIV AB",           "code": "84",               "use": ""                         },
    {"mnemonic": "MOV p,q",          "code": "85qqpp",           "use": "p=data_w,q=data_r"        },
    {"mnemonic": "MOV p,@R0",        "code": "86pp",             "use": "p=data_w"                 },
    {"mnemonic": "MOV p,@R1",        "code": "87pp",             "use": "p=data_w"                 },
    {"mnemonic": "MOV p,R0",         "code": "88pp",             "use": "p=data_w"                 },
    {"mnemonic": "MOV p,R1",         "code": "89pp",             "use": "p=data_w"                 },
    {"mnemonic": "MOV p,R2",         "code": "8App",             "use": "p=data_w"                 },
    {"mnemonic": "MOV p,R3",         "code": "8Bpp",             "use": "p=data_w"                 },
    {"mnemonic": "MOV p,R4",         "code": "8Cpp",             "use": "p=data_w"                 },
    {"mnemonic": "MOV p,R5",         "code": "8Dpp",             "use": "p=data_w"                 },
    {"mnemonic": "MOV p,R6",         "code": "8Epp",             "use": "p=data_w"                 },
    {"mnemonic": "MOV p,R7",         "code": "8Fpp",             "use": "p=data_w"                 },

    {"mnemonic": "MOV DPTR,#b",      "code": "90bb",             "use": "b=const"                  },
    {"mnemonic": "ACALL p",          "code": "91pp",             "use": "p=code_11"                },
    {"mnemonic": "MOV b,C",          "code": "92bb",             "use": "b=data_bit_w"             },
    {"mnemonic": "MOVC A,@A+DPTR",   "code": "93",               "use": ""                         },
    {"mnemonic": "SUBB A,#b",        "code": "94bb",             "use": "b=const"                  },
    {"mnemonic": "SUBB A,p",         "code": "95pp",             "use": "p=data_r"                 },
    {"mnemonic": "SUBB A,@R0",       "code": "96",               "use": ""                         },
    {"mnemonic": "SUBB A,@R1",       "code": "97",               "use": ""                         },
    {"mnemonic": "SUBB A,R0",        "code": "98",               "use": ""                         },
    {"mnemonic": "SUBB A,R1",        "code": "99",               "use": ""                         },
    {"mnemonic": "SUBB A,R2",        "code": "9A",               "use": ""                         },
    {"mnemonic": "SUBB A,R3",        "code": "9B",               "use": ""                         },
    {"mnemonic": "SUBB A,R4",        "code": "9C",               "use": ""                         },
    {"mnemonic": "SUBB A,R5",        "code": "9D",               "use": ""                         },
    {"mnemonic": "SUBB A,R6",        "code": "9E",               "use": ""                         },
    {"mnemonic": "SUBB A,R7",        "code": "9F",               "use": ""                         },

    {"mnemonic": "ORL C,/b",         "code": "A0bb",             "use": "b=data_bit_r"             },
    {"mnemonic": "AJMP p",           "code": "A1pp",             "use": "p=code_11"                },
    {"mnemonic": "MOV C,b",          "code": "A2bb",             "use": "b=data_bit_r"             },
    {"mnemonic": "INC DPTR",         "code": "A3",               "use": ""                         },
    {"mnemonic": "MUL AB",           "code": "A4",               "use": ""                         },
    {"mnemonic": "MOV @R0,p",        "code": "A6",               "use": ""                         },
    {"mnemonic": "MOV @R1,p",        "code": "A7",               "use": ""                         },
    {"mnemonic": "MOV R0,p",         "code": "A8",               "use": ""                         },
    {"mnemonic": "MOV R1,p",         "code": "A9",               "use": ""                         },
    {"mnemonic": "MOV R2,p",         "code": "AA",               "use": ""                         },
    {"mnemonic": "MOV R3,p",         "code": "AB",               "use": ""                         },
    {"mnemonic": "MOV R4,p",         "code": "AC",               "use": ""                         },
    {"mnemonic": "MOV R5,p",         "code": "AD",               "use": ""                         },
    {"mnemonic": "MOV R6,p",         "code": "AE",               "use": ""                         },
    {"mnemonic": "MOV R7,p",         "code": "AF",               "use": ""                         },

    {"mnemonic": "ANL C,/b",         "code": "B0bb",             "use": "b=data_bit_r"             },
    {"mnemonic": "ACALL p",          "code": "B1pp",             "use": "p=code_11"                },
    {"mnemonic": "CPL b",            "code": "B2bb",             "use": "b=data_bit_rw"            },
    {"mnemonic": "CPL C",            "code": "B3",               "use": ""                         },
    {"mnemonic": "CJNE A,#b,r",      "code": "B4bbrr",           "use": "b=const,r=code_pcr"       },
    {"mnemonic": "CJNE A,p,r",       "code": "B5pprr",           "use": "p=data_r,r=code_pcr"      },
    {"mnemonic": "CJNE @R0,#b,r",    "code": "B6bbrr",           "use": "b=const,r=code_pcr"       },
    {"mnemonic": "CJNE @R1,#b,r",    "code": "B7bbrr",           "use": "b=const,r=code_pcr"       },
    {"mnemonic": "CJNE R0,#b,r",     "code": "B8bbrr",           "use": "b=const,r=code_pcr"       },
    {"mnemonic": "CJNE R1,#b,r",     "code": "B9bbrr",           "use": "b=const,r=code_pcr"       },
    {"mnemonic": "CJNE R2,#b,r",     "code": "BAbbrr",           "use": "b=const,r=code_pcr"       },
    {"mnemonic": "CJNE R3,#b,r",     "code": "BBbbrr",           "use": "b=const,r=code_pcr"       },
    {"mnemonic": "CJNE R4,#b,r",     "code": "BCbbrr",           "use": "b=const,r=code_pcr"       },
    {"mnemonic": "CJNE R5,#b,r",     "code": "BDbbrr",           "use": "b=const,r=code_pcr"       },
    {"mnemonic": "CJNE R6,#b,r",     "code": "BEbbrr",           "use": "b=const,r=code_pcr"       },
    {"mnemonic": "CJNE R7,#b,r",     "code": "BFbbrr",           "use": "b=const,r=code_pcr"       },

    {"mnemonic": "PUSH p",           "code": "C0pp",             "use": "p=data_r"                 },
    {"mnemonic": "AJMP p",           "code": "C1pp",             "use": "p=code_11"                },
    {"mnemonic": "CLR b",            "code": "C2bb",             "use": "b=data_bit_w"             },
    {"mnemonic": "CLR C",            "code": "C3",               "use": ""                         },
    {"mnemonic": "SWAP A",           "code": "C4",               "use": ""                         },
    {"mnemonic": "XCH A,p",          "code": "C5pp",             "use": "p=data_rw"                },
    {"mnemonic": "XCH A,@R0",        "code": "C6",               "use": ""                         },
    {"mnemonic": "XCH A,@R1",        "code": "C7",               "use": ""                         },
    {"mnemonic": "XCH A,R0",         "code": "C8",               "use": ""                         },
    {"mnemonic": "XCH A,R1",         "code": "C9",               "use": ""                         },
    {"mnemonic": "XCH A,R2",         "code": "CA",               "use": ""                         },
    {"mnemonic": "XCH A,R3",         "code": "CB",               "use": ""                         },
    {"mnemonic": "XCH A,R4",         "code": "CC",               "use": ""                         },
    {"mnemonic": "XCH A,R5",         "code": "CD",               "use": ""                         },
    {"mnemonic": "XCH A,R6",         "code": "CE",               "use": ""                         },
    {"mnemonic": "XCH A,R7",         "code": "CF",               "use": ""                         },

    {"mnemonic": "POP p",            "code": "D0pp",             "use": "p=data_w"                 },
    {"mnemonic": "ACALL p",          "code": "D1pp",             "use": "p=code_11"                },
    {"mnemonic": "SETB b",           "code": "D2bb",             "use": "b=data_bit_w"             },
    {"mnemonic": "SETB C",           "code": "D3",               "use": ""                         },
    {"mnemonic": "DA A",             "code": "D4",               "use": ""                         },
    {"mnemonic": "DJNZ p,r",         "code": "D5pprr",           "use": "p=data_rw,r=code_pcr"     },
    {"mnemonic": "XCHD A,@R0",       "code": "D6",               "use": ""                         },
    {"mnemonic": "XCHD A,@R1",       "code": "D7",               "use": ""                         },
    {"mnemonic": "DJNZ R0,r",        "code": "D8rr",             "use": "r=code_pcr"               },
    {"mnemonic": "DJNZ R1,r",        "code": "D9rr",             "use": "r=code_pcr"               },
    {"mnemonic": "DJNZ R2,r",        "code": "DArr",             "use": "r=code_pcr"               },
    {"mnemonic": "DJNZ R3,r",        "code": "DBrr",             "use": "r=code_pcr"               },
    {"mnemonic": "DJNZ R4,r",        "code": "DCrr",             "use": "r=code_pcr"               },
    {"mnemonic": "DJNZ R5,r",        "code": "DDrr",             "use": "r=code_pcr"               },
    {"mnemonic": "DJNZ R6,r",        "code": "DErr",             "use": "r=code_pcr"               },
    {"mnemonic": "DJNZ R7,r",        "code": "DFrr",             "use": "r=code_pcr"               },

    {"mnemonic": "MOVX A,@DPTR",     "code": "E0",               "use": ""                         },
    {"mnemonic": "AJMP p",           "code": "E1pp",             "use": "p=code_11"                },
    {"mnemonic": "MOVX A,@R0",       "code": "E2",               "use": ""                         },
    {"mnemonic": "MOVX A,@R1",       "code": "E3",               "use": ""                         },
    {"mnemonic": "CLR A",            "code": "E4",               "use": ""                         },
    {"mnemonic": "MOV A,p",          "code": "E5pp",             "use": "p=data_r"                 },
    {"mnemonic": "MOV A,@R0",        "code": "E6",               "use": ""                         },
    {"mnemonic": "MOV A,@R1",        "code": "E7",               "use": ""                         },
    {"mnemonic": "MOV A,R0",         "code": "E8",               "use": ""                         },
    {"mnemonic": "MOV A,R1",         "code": "E9",               "use": ""                         },
    {"mnemonic": "MOV A,R2",         "code": "EA",               "use": ""                         },
    {"mnemonic": "MOV A,R3",         "code": "EB",               "use": ""                         },
    {"mnemonic": "MOV A,R4",         "code": "EC",               "use": ""                         },
    {"mnemonic": "MOV A,R5",         "code": "ED",               "use": ""                         },
    {"mnemonic": "MOV A,R6",         "code": "EE",               "use": ""                         },
    {"mnemonic": "MOV A,R7",         "code": "EF",               "use": ""                         },

    {"mnemonic": "MOVX @DPTR,A",     "code": "F0",               "use": ""                         },
    {"mnemonic": "ACALL p",          "code": "F1pp",             "use": "p=code_11"                },
    {"mnemonic": "MOVX @R0,A",       "code": "F2",               "use": ""                         },
    {"mnemonic": "MOVX @R1,A",       "code": "F3",               "use": ""                         },
    {"mnemonic": "CPL A",            "code": "F4",               "use": ""                         },
    {"mnemonic": "MOV p,A",          "code": "F5pp",             "use": "p=data_w"                 },
    {"mnemonic": "MOV @R0,A",        "code": "F6",               "use": ""                         },
    {"mnemonic": "MOV @R1,A",        "code": "F7",               "use": ""                         },
    {"mnemonic": "MOV R0,A",         "code": "F8",               "use": ""                         },
    {"mnemonic": "MOV R1,A",         "code": "F9",               "use": ""                         },
    {"mnemonic": "MOV R2,A",         "code": "FA",               "use": ""                         },
    {"mnemonic": "MOV R3,A",         "code": "FB",               "use": ""                         },
    {"mnemonic": "MOV R4,A",         "code": "FC",               "use": ""                         },
    {"mnemonic": "MOV R5,A",         "code": "FD",               "use": ""                         },
    {"mnemonic": "MOV R6,A",         "code": "FE",               "use": ""                         },
    {"mnemonic": "MOV R7,A",         "code": "FF",               "use": ""                         },
]


class CPU_8052(opcodetools.cpu.base_cpu.CPU):

    def __init__(self):
        super().__init__(OPCODES)
